
/*****************************************************************************/
/*                                                                           */
/*                	National Instruments GPIB-VXI Project                    */
/*						DownLoable Code Instruments							 */
/*                                                                           */
/*                     (C) 1989  National Instruments                        */
/*                                                                           */
/*****************************************************************************/

	/* Word offsets into the VXI register space for a particular register */
#define IdReg				0
#define DeviceTypeReg		1
#define StatusReg			2
#define ProtocolReg			4
#define ResponseReg			5


	/* Id register fields */
#define DCI_ManufacturerId		0xff6			/* National Instruments */
#define DCI_AddressSpace		(0x3<<12)		/* A16 only */
#define DEV_CLASS_MASK			0xc000
#define DEV_CLASS_MBD			2						/* Message Base Device Class Value */
#define	DCI_DeviceClass			(DEV_CLASS_MBD<<14)		/* DCI is a Message Based Device	*/

	/* Device Type fields */
#define DCI_ModelCode			0xdc1			/* Choose one */
#define DCI_RequiredMemory		(0xf<<12)		/* none--Always leave this value!!! */

	/* Status Register bits */
#define Status_PASSED		(1<<2)		/* Self-test passed */
#define Status_READY		(1<<3)		/* Ready to execute normal operation commands */

	/* Protocol Register bits */
#define Protocol_CMDR		(1<<15)		/* 0 = both commander/servant capability */ 
#define Protocol_SIGNAL		(1<<14)		/* 0 = has signal register */ 
#define Protocol_MASTER		(1<<13)		/* 0 = has VMEbus master cap.*/ 
#define Protocol_INTER		(1<<12)		/* 1 = has interrupter cap. */ 
#define Protocol_FHS		(1<<11)		/* 0 = has fast handshake cap.  */ 
#define Protocol_SHAREDMEM	(1<<10)		/* 0 = has shared memory cap.  */ 

	/* Response Register Bits */
#define Response_RESERVED	(1<<14)		/* 1 always, reserved */ 
#define Response_DOR		(1<<13)		/* 1 = ready to output data */ 
#define Response_DIR		(1<<12)		/* 1 = ready to input data */ 
#define Response_ERR		(1<<11)		/* 0 = error unreported */ 
#define Response_RREADY		(1<<10)		/* 1 = data low has data in it*/ 
#define Response_WREADY		(1<<9)		/* 1 = ready for write to data low*/ 
#define Response_FHS		(1<<8)		/* 0 = fast hanshake mode on */ 
#define Response_LOCKED		(1<<7)		/* 0 = access locked to local sources*/ 


	/** DCI Asynch Exchange events **/
#define DCI_EVENT_ServantFailed		0
#define DCI_EVENT_WS_Data			1
#define DCI_EVENT_WS_Command		2
#define DCI_EVENT_Signal			3
#define DCI_EVENT_Control_In		4
#define DCI_EVENT_WSphys_Command	5
#define DCI_EVENT_DOR_Request		6



	 /**************************************/
	 /*        DCI Header Definition	   */
	 /**************************************/

typedef struct DCI_node {
	uint16	VXIregs[0x20];				/* Software version of Applicable VXI registers*/

		/* Library Function jump table addresses--MUST be at DCI_node offset 0x40!!!! */
	uint32	pSOSTablePointer;			/* Address of pSOS function jump table 		  */
	uint32	pREPcTablePointer;			/* Address of pREPc function jump table 	  */
	uint32	NITablePointer;				/* Address of National Instruments jump table */
	uint32	NI2TablePointer;			/* Address of National Instruments jump table */
	uint32	ReservedTablePointer[4];	/* Reserved function table pointer			  */

		/* DCI User defined Function Tables */
	uint32	DCI_FunctionTable[5];

		/* INTERNAL USE GPIB-VXI Variables */
	uint32	ReservedINTERNAL[12];

		/* Miscellaneous Variables */
	int8	PropagateBNO;				/* Flag to tell DCI whether to propagate BNO  */
	int8	SoftReset;					/* Boolean to tell whether coming out of SoftR*/
	int8	Debug;						/* tells whether CI was booted with DEBUG on  */
	int8    RORAhandler;                /* bit-wise flags--which int handlers are RORA*/
	uint32	*GPIB_VXI_EventEnables;		/* ptr to 4 longword array of enabled events  */
	uint16  *GPIB_VXI_ProtocolError;    /* ptr to GPIB-VXI Protocol Error value */

		/* Processes, Exchange, and Parser Identification numbers */
	uint32	WorkerPid;					/* Worker Process pSOS Identification Number  */
	uint32	AsynchPid;					/* Asynch Process pSOS Identification Number  */
	uint32	AsynchXid;					/* Asynch Exchange pSOS Identification Number */
	uint16	ParserId;					/* Value used by Parser to identify DCI link  */

		/* DCI Logical Address information */
	int8	DCILogicalAddress;			/* This DCI's Logical Address				  */
	int8	CmdrLogicalAddress;			/* Logical Address of DCI's Commander    	  */

		/* GPIB-VXI and Resource Manager Table Information */
	int8	F1;							/* Filler for word alignment 				  */
	int8	GPIB_VXI_LA;				/* The GPIB-VXI's Logical Address (if needed) */
	int8	*laValid;					/* boolean array of phys devices (ex. GPIB-VXI) */
	int8	*laDCI;						/* boolean array of DCI's on this GPIB-VXI	  */
	int8	*CMDRtable;					/* Array of commanders for each log addr	  */
	la_entry	*la_table;				/* Resource Manager Table information		  */
	int8	*ServantList;				/* Servant Lists pointed to by la_table		  */

		/* GPIB-VXI Dual Port memory info */
	uint32	DPVbase;					/* Base address of DP RAM (as seen from VXI bus)  */
	uint32	DPLbase;					/* Base address of DP RAM (as seen locally)   */
	uint32	DPsize;						/* size (in bytes) of DP RAM on this GPIB-VXI */

		/* DCI memory usage */
	uint32	DCIGlobalBase;				/* Base of all DCI RAM Blocks in system		  */
	uint16	StartingBlock;				/* Starting block of this DCI (as requested)  */
	uint16	NumberOfBlocks;				/* Number of 4K blocks requested			  */

		/* Variables DCI pokes in */
	void	(*WorkerStart)();			/* Starting address of Worker process		  */
	void	(*AsynchStart)();			/* Starting address of Asynch process		  */
	uint16	StatusByte;					/* Current VXI (and 488.2) Status byte 		  */
	uint16	AsynMode;					/* Current Asynchronous Mode Control condition*/
	uint16	RespEna;					/* Current Response Enable condition		  */


		/* DCI User defined space follows. (make sure allocate space!) */

} DCI_node;



