/*********************************************************************/ 
/*                                                                   */ 
/*          Copyright 1985, Software Components Group, Inc.          */ 
/*                      ALL RIGHTS RESERVED                          */ 
/*                                                                   */ 
/*   This computer program is the property of Software Components    */ 
/*   Group Inc., Santa Clara, Calif., U.S.A. and may not be copied   */ 
/*   in any form or by any means, whether in part or in whole,       */ 
/*   except under license expressly granted by Software Components   */ 
/*   Group Inc.                                                      */ 
/*                                                                   */ 
/*   All copies of this program, whether in part or in whole, and    */ 
/*   whether modified or not, must display this and all other        */ 
/*   embedded copyright and ownership notices in full.               */ 
/*                                                                   */ 
/*-------------------------------------------------------------------*/
/*                                                                   */
/*   Copyright 1984, Green Hills Software, Inc.  This program is     */
/*   the property of Green Hills Software, Inc, its contents are     */
/*   proprietary information and no part of it is to be disclosed    */
/*   to anyone except employees of Green Hills Software, Inc., or    */
/*   as agreed to in writing signed by the President of Green Hills  */
/*   Software, Inc.                                                  */
/*                                                                   */
/*********************************************************************/
  
#define NULL    0
#define EOF     (-1)
#define ERROR   (-2)
#define BUFSIZ  512     /* buffer size for all IO buffers */
#define BLKSIZ  BUFSIZ
  
#define IOIDENT 0x12345678
  
#define _NFILE  20
  
typedef struct
    { 
    char *io_next;  /* next character to be read from buffer, next
                        position to put a character in buffer */
    char *io_lid;   /* last position with data in input buffer */ 
    char *io_top;   /* position beyond buffer */
    char *io_base;  /* start of buffer */ 
    short io_file;  /* location in iob array, also unix style file # */ 
    short io_flags; /* bit fields, defined below */ 
    short io_tmp;   /* used by ungetc on unbuffered files */
    int io_id;      /* fid returned by pHILE, or pSOS device # */ 
    } FILE; 
  
#define _IO_READ        0x001  /* open for read */
#define _IO_WRITE       0x002  /* open for write */ 
#define _IO_ERROR       0x004  /* some error has happenned */ 
#define _IO_EOF         0x008  /* file is positioned to end-of-file */
#define _IO_APPEND      0x010  /* file is opened in "append mode" */
#define _IO_pHILE       0x020  /* file is opened under pHILE */ 
#define _IO_BUF         0x040  /* has a system supplied buffer */ 
#define _IO_USRBUF      0x080  /* has a user supplied buffer */ 
#define _IO_STR         0x100  /* used by sprintf, maybe */ 
  
#define SEEK_SET        0 
#define SEEK_CUR        1 
#define SEEK_END        2 
  
#define _OP_EOF 0x1000
  
#define fileno(file)    ((file)->io_file) 
#define feof(file)      ((file)->io_flags&_IO_EOF)
#define ferror(file)    ((file)->io_flags&_IO_ERROR)
#define clearerr(file)  (file)->io_flags&=~(_IO_EOF|_IO_ERROR)
  
#define errno (get_pcb_io()->lerrno)
  
typedef struct free_hdr 
    { 
    long length;      /* Size of segment in units of FREE's */
    struct free_hdr *next;  /* Pointer to next item in free list */ 
    } FREE; 
  
#define S_SIZE 100
struct pcb_io 
    { 
    char *next;       /* ptr to another PCB extension, if any */
    long ident;       /* verifies that this is pcb_io structure */
    int lerrno;       /* error #, similar to UNIX errno */
    FILE iob[_NFILE]; /* control info for each open file */ 
    FREE *memfirst;   /* ptr to head of free list of memory segments */ 
    FREE *memnext;    /* used in memory allocation to keep Knuth happy*/
    char space[S_SIZE]; /* scratch space for print formatting functions */
    long seed[2];     /* for random number generator */ 
    int initted;      /*  "    "      "        "     */ 
    };
  
struct pcb
    { 
    char filler[138];
    struct pcb_io *ext; 
    };
  
struct pcb_io *get_pcb_io();
FILE *f_addr(); 
  
#define stdin   (f_addr(0)) 
#define stdout  (f_addr(1)) 
#define stderr  (f_addr(2)) 
  

#define getc(f) ((f->io_next > f->io_lid) ? _filbuf(f) : \
  (unsigned) *(f->io_next++))
#define getchar() getc(stdin)

#define putc(ch,f) (((f->io_tmp=ch)=='\n' || f->io_next >= f->io_top) ? \
  _flsbuf(f,f->io_tmp):(*(f->io_next++) = f->io_tmp))
#define putb(ch,f) (((f)->io_next < (f)->io_top) ? (*((f)->io_next++) = ch) :\
  _flsbuf(f,ch))
#define putchar(ch) putc(ch,stdout)

FILE *fopen();
char *gets(), *fgets();
long getl();
short getw();
