
/*****************************************************************************/
/*                                                                           */
/*                	National Instruments GPIB-VXI Project                    */
/*						DownLoable Code Instruments							 */
/*                                                                           */
/*                     (C) 1989  National Instruments                        */
/*                                                                           */
/*****************************************************************************/
/*
 * The following are the possible bit-wise values for the parameter 'mode'
 * in the 'DMAmove' function.  Great care should be used if Burst mode transfers
 * are used.  The entire time a burst transfer in going on the CPU cannot get
 * access to ANY resoureces (i.e. RAM, EPROM, I/O).  This effectively HALTS execution
 * of every other process in the system while the transfer is going on.
 */


/*
 * In the following descriptors:    Source = MEMORY,   Destination = DEVICE
 *
 *	NOTE: a zero in the mode field means a 16 bit, memory to device transfer, cycle steal,
 *		incrementing source and destination.
 */


#define DMA_MODE_DevToMem		(1<<0)	/* transfer from device to memory */
#define DMA_MODE_MemToDev		0		/* transfer from memory to device */


#define DMA_MODE_8bitDev		(1<<1)	/* 8 bit wide device (perform only D08 transfers) */
										/* NOTE: Destination addr A0bit = upper or lower byte */
#define DMA_MODE_16bitDev		0		/* 16 bit wide device (perform only D16 transfers) */


#define DMA_MODE_8bitOper		(1<<2)	/* 8 bit wide Operand: (inc addresses by 1) */
#define DMA_MODE_16bitOper		0		/* 16 bit wide Operand: (inc addresses by 2) 
											if 8 bit device do two 8 bit transfers 
											if 16 bit device do one 16 bit transfer) */


#define DMA_MODE_Burst			(1<<3)	/* hold processor bus while transferring */
#define DMA_MODE_CycleSteal		0		/* give up processor bus after each transfer */


#define DMA_MODE_NoIncSRC		(1<<4)	/* keep source address same after each transfer */
#define DMA_MODE_IncSRC			0		/* Increment source address after each transfer */


#define DMA_MODE_NoIncDEST		(1<<5)	/* keep dest address same after each transfer */
#define DMA_MODE_IncDEST		0		/* Increment dest address after each transfer */



	/* 'DMAmove' function return codes */
#define DMA_RET_NO_ERR          0		/* transfer successful */
#define DMA_RET_TIME_ERR        2		/* a DMA timing error occurred */
#define DMA_RET_BUS_ERR_MEM     9		/* the source memory address bus errored */
#define DMA_RET_BUS_ERR_DEV     10		/* the destination mem address bus errored */

