
*****************************************************************************
*
*				   National Instruments  GPIB-VXI Project
*					 (c) 1989  All Rights Reserved.
*
*
*		    	    Sample format of Reserved DCI Space
*
*
*	The following is a file to reserve space for the DCI Header
* area at the base of the DCI object file.  It is in Assembly to guarentee 
* that it will always be put at the base of the object file.
*
* IMPORTANT NOTE:
*	Since it is allocated in Section 0 and all other code segments and data
* segments are allocated in higher number sections, it will be put first in
* the object file.  If the Green Hill C Compiler is not used, the programmer
* must find a way to have the object of this file put FIRST in the final
* object file.  All other object code may reside anywhere.
*
*****************************************************************************

*
*	The include file included in this section is created automatically by
* the makefile.  It contains a single equate defining the variable 
* 'CIStartBlock'.  This equate is used to change the base block which an
* Epromable Code Instrument (ECI) will be placed at.
*
*			INCLUDE	StartBlock.inc

*****************************************************************************

			SECTION 0

* pre-Init routine will fill in the values, so just allocated the space for
* it and we are done.
* In the C files this block is type cast to struct type 'DCI_node' and 
* used as the DCI Header.


			xdef	DCIheader
DCIheader:
			ds.b	$100

*********************************************************************************
*
*	The following code consists of two boot entry points for code instruments.
* The first, the pre-init entry, is used by all types of code instruments. 
* The second, the setup structure, is used only by epromed code instruments.
* The setup structure is basically a copy of the parameters that would have been
* sent in under the DCISetUp? query.
*
*	The relative offsets into the code is used to find these vectors, so its
* location is EXTREMLY important.  For downloaded code instruments, these
* vectors are assumed to be at offset 0x100 of the file (immediately after
* the header structure).  For other code instruments, these vecotrs are assumed
* to be at offset 0x000 of the code section (since the code section is no longer
* at the same place as the data and header).
*	Since these offsets are different under the two cases, they have been placed
* in different sections (0 and 1).  For downloaded code instruments, all
* sections can be compiled in order as is.  For other code instruments, section 0
* (the Header) should be moved, using the -O0:xADDR link directive, to the base of 
* the requested Data blocks for the code instrument.  Then section 14 (uninitialized
* data or variable data) should be moved to the base + 0x100 of the requested Data
* blocks using the -O14:x(ADDR+0x100) link directive.
*
			SECTION 1

			xref	DCIpreInit
InitVector	bra.w	DCIpreInit


CISetUpData:

YES			equ		0
NO			equ		1
PosIndCI	equ		YES
			ifeq	PosIndCI

CIlogaddr	dc.b	$ff				* Logical Address to put CI at ($FF = dynamic or self-assign)
CmdrLogAddr	dc.b	$ff				* Commander's Logical Address ($FF = this GPIB-VXI)
StartBlock	dc.w	$ffff			* FFFF = flag for PI CI  (not Starting Ram Block to use)
NumBlocks	dc.w	$ffff			* UNUSED FIELD  (not Number of 4K Ram Blocks needed)
DynamicRAM	dc.l	$00003000		* number of bytes of dynamic RAM needed
WorkerStack	dc.w	$0400			* Number of WORDS in Worker Stack (max is 128K bytes)
Servants	dc.b	$ff				* List of initial Servants Ending with $FF
			dc.b	$ff				*       (always word align code and data!!!)

			endc

			ifne	PosIndCI

CIlogaddr	dc.b	$ff				* Logical Address to put CI at ($FF = dynamic or self-assign)
CmdrLogAddr	dc.b	$ff				* Commander's Logical Address ($FF = this GPIB-VXI)
StartBlock	dc.w	CIStartBlock	* Starting Ram Block to use
NumBlocks	dc.w	$0001			* Number of 4K Ram Blocks needed (only for data segment)
WorkerStack	dc.w	$0400			* Number of WORDS in Worker Stack (max is 128K bytes)
Servants	dc.b	$ff				* List of initial Servants Ending with $FF
			dc.b	$ff				*       (always word align code and data!!!)

			endc

*********************************************************************************
			end

