
***************************************************************************
*
*				National Instruments GPIB-VXI Project
*					 DCI Library Function Caller 
*
*                  (c) 1989  National Instruments
*
***************************************************************************
*	The following file contains all of the pSOS, pREPc, and National
* Instruments function names that can be called by a DCI.
*	The actual functions are held in the NI EPROM which, upon creation of the
* DCI, pokes in the address of the jump tables to these functions into the
* DCI Header.
*	This file simply provides the names so they look like the C functions
* are being included with the DCI code when they are actually already in
* the EPROM.  This interface converts the C function call to a lookup in the 
* function pointer table and a jumps to that function.
*   This leads to very small DCI's and a common control (namely National 
* Instruments) over the way that functions are performed.
***************************************************************************

* put in separate section so you can see exactly who much space it takes
					SECTION 11


YES			equ		0
NO			equ		1
PosIndCI	equ		YES		** CHANGE THIS DEPENDING ON COMPILER TYPE **



TableBase			equ		$40		* Static base of jump tables in Header
OffsetSize			equ		$04		* Addresses are longwords

pSOSTableOffset		equ		0
pREPcTableOffset	equ		1
NITableOffset		equ		2
NI2TableOffset		equ		3

					xref	DCIheader


***************************************************************************
*						pSOS System Calls
***************************************************************************

N_spawn_p		equ		0*OffsetSize
N_activate_p	equ		1*OffsetSize
N_ident_p		equ		2*OffsetSize
N_delete_p		equ		3*OffsetSize
N_suspend_p		equ		4*OffsetSize
N_resume_p		equ		5*OffsetSize
N_mode_p		equ		6*OffsetSize
N_pause_p		equ		7*OffsetSize
N_get_t			equ		8*OffsetSize
N_create_x		equ		9*OffsetSize
N_attach_x		equ		10*OffsetSize
N_delete_x		equ		11*OffsetSize
N_send_x		equ		12*OffsetSize
N_jam_x			equ		13*OffsetSize
N_req_x			equ		14*OffsetSize
N_signal_v		equ		15*OffsetSize
N_wait_v		equ		16*OffsetSize
N_alloc_seg		equ		17*OffsetSize
N_free_seg		equ		18*OffsetSize
N_assign_seg	equ		19*OffsetSize
N_grab_seg		equ		20*OffsetSize
N_d_init		equ		21*OffsetSize
N_d_open		equ		22*OffsetSize
N_d_close		equ		23*OffsetSize
N_d_read		equ		24*OffsetSize
N_d_write		equ		25*OffsetSize
N_d_ctrl		equ		26*OffsetSize
N_priority_p	equ		27*OffsetSize
N_create_pt		equ		28*OffsetSize
N_delete_pt		equ		29*OffsetSize
N_get_buf		equ		30*OffsetSize
N_ret_buf		equ		31*OffsetSize
N_liberate_x	equ		32*OffsetSize


				xdef	spawn_p 
				xdef	activate_p
				xdef	ident_p
				xdef	delete_p 
				xdef	suspend_p
				xdef	resume_p
				xdef	mode_p
				xdef	pause_p
				xdef	get_t
				xdef	create_x
				xdef	attach_x
				xdef	delete_x
				xdef	send_x
				xdef	jam_x
				xdef	req_x
				xdef	signal_v
				xdef	wait_v	
				xdef	alloc_seg
				xdef	free_seg
				xdef	assign_seg
				xdef	grab_seg
				xdef	d_init
				xdef	d_open
				xdef	d_close
				xdef	d_read
				xdef	d_write
				xdef	d_ctrl	
				xdef	priority_p
				xdef	create_pt
				xdef	delete_pt
				xdef	get_buf
				xdef	ret_buf
				xdef	liberate_x


spawn_p			move.l	#N_spawn_p,D0
				bra		Handle_pSOS_Call
activate_p		move.l	#N_activate_p,D0
				bra		Handle_pSOS_Call
ident_p			move.l	#N_ident_p,D0
				bra.s	Handle_pSOS_Call
delete_p		move.l	#N_delete_p,D0
				bra.s	Handle_pSOS_Call
suspend_p		move.l	#N_suspend_p,D0
				bra.s	Handle_pSOS_Call
resume_p		move.l	#N_resume_p,D0
				bra.s	Handle_pSOS_Call
mode_p			move.l	#N_mode_p,D0
				bra.s	Handle_pSOS_Call
pause_p			move.l	#N_pause_p,D0
				bra.s	Handle_pSOS_Call
get_t			move.l	#N_get_t,D0
				bra.s	Handle_pSOS_Call
create_x		move.l	#N_create_x,D0
				bra.s	Handle_pSOS_Call
attach_x		move.l	#N_attach_x,D0
				bra.s	Handle_pSOS_Call
delete_x		move.l	#N_delete_x,D0
				bra.s	Handle_pSOS_Call
send_x			move.l	#N_send_x,D0
				bra.s	Handle_pSOS_Call
jam_x			move.l	#N_jam_x,D0
				bra.s	Handle_pSOS_Call
req_x			move.l	#N_req_x,D0
				bra.s	Handle_pSOS_Call
signal_v		move.l	#N_signal_v,D0
				bra.s	Handle_pSOS_Call
wait_v			move.l	#N_wait_v,D0
				bra.s	Handle_pSOS_Call
alloc_seg		move.l	#N_alloc_seg,D0
				bra.s	Handle_pSOS_Call
free_seg		move.l	#N_free_seg,D0
				bra.s	Handle_pSOS_Call
assign_seg		move.l	#N_assign_seg,D0
				bra.s	Handle_pSOS_Call
grab_seg		move.l	#N_grab_seg,D0
				bra.s	Handle_pSOS_Call
d_init			move.l	#N_d_init,D0
				bra.s	Handle_pSOS_Call
d_open			move.l	#N_d_open,D0
				bra.s	Handle_pSOS_Call
d_close			move.l	#N_d_close,D0
				bra.s	Handle_pSOS_Call
d_read			move.l	#N_d_read,D0
				bra.s	Handle_pSOS_Call
d_write			move.l	#N_d_write,D0
				bra.s	Handle_pSOS_Call
d_ctrl			move.l	#N_d_ctrl,D0
				bra.s	Handle_pSOS_Call
priority_p		move.l	#N_priority_p,D0
				bra.s	Handle_pSOS_Call
create_pt		move.l	#N_create_pt,D0
				bra.s	Handle_pSOS_Call
delete_pt		move.l	#N_delete_pt,D0
				bra.s	Handle_pSOS_Call
get_buf			move.l	#N_get_buf,D0
				bra.s	Handle_pSOS_Call
ret_buf			move.l	#N_ret_buf,D0
				bra.s	Handle_pSOS_Call
liberate_x		move.l	#N_liberate_x,D0

*				bra.s	Handle_pSOS_Call		* last branch not needed

*****************************************************************************

pSOSTablePointer	equ		(TableBase+(pSOSTableOffset*OffsetSize))

Handle_pSOS_Call:
	ifne	PosIndCI
				lea		DCIheader,A0		*get pointer to header
	endc
	ifeq	PosIndCI
				lea		DCIheader(A5),A0	*get pointer to header (PI code)
	endc
				adda.l	#pSOSTablePointer,A0
				move.l	(A0),A0				*get pointer to Jump Table
				move.l	0(A0,D0.W),A0		*get pointer to function
				jmp		(A0)				*jump to function



***************************************************************************
*						pREPc System Calls
***************************************************************************

N_start			equ		0*OffsetSize
N_exit			equ		1*OffsetSize
N_fclose		equ		2*OffsetSize
N_fflush		equ		3*OffsetSize
N_fgetc			equ		4*OffsetSize
N_fgets			equ		5*OffsetSize
N_fopen			equ		6*OffsetSize
N_fputc			equ		7*OffsetSize
N_fputs			equ		8*OffsetSize
N_fread			equ		9*OffsetSize
N_fseek			equ		10*OffsetSize
N_fwrite		equ		11*OffsetSize
N_printf		equ		12*OffsetSize
N_scanf			equ		13*OffsetSize
N_sscanf		equ		14*OffsetSize
N_sprintf		equ		15*OffsetSize
N_gets			equ		16*OffsetSize
N_f_addr		equ		17*OffsetSize
N_filbuf		equ		18*OffsetSize
N_flsbuf		equ		19*OffsetSize
N_strcpy		equ		20*OffsetSize
N_strcmp		equ		21*OffsetSize
N_strlen		equ		22*OffsetSize
N_puts			equ		23*OffsetSize
N_strncat		equ		24*OffsetSize
N_strncmp		equ		25*OffsetSize
N_strncpy		equ		26*OffsetSize
N_strrchr		equ		27*OffsetSize

				xdef	start
				xdef	exit
				xdef	fclose
				xdef	fflush
				xdef	fgetc
				xdef	fgets
				xdef	fopen
				xdef	fputc
				xdef	fputs
				xdef	fread
				xdef	fseek
				xdef	fwrite
				xdef	printf
				xdef	scanf
				xdef	sscanf
				xdef	sprintf
				xdef	gets
				xdef	f_addr
				xdef	.filbuf	
				xdef	.flsbuf
				xdef	strcpy
				xdef	strcmp
				xdef	strlen
				xdef	puts
				xdef	strncat
				xdef	strncmp
				xdef	strncpy
				xdef	strrchr


start			move.l	#N_start,D0
				bra.s	Handle_pREPc_Call
exit			move.l	#N_exit,D0
				bra.s	Handle_pREPc_Call
fclose			move.l	#N_fclose,D0
				bra.s	Handle_pREPc_Call
fflush			move.l	#N_fflush,D0
				bra.s	Handle_pREPc_Call
fgetc			move.l	#N_fgetc,D0
				bra.s	Handle_pREPc_Call
fgets			move.l	#N_fgets,D0
				bra.s	Handle_pREPc_Call
fopen			move.l	#N_fopen,D0
				bra.s	Handle_pREPc_Call
fputc			move.l	#N_fputc,D0
				bra.s	Handle_pREPc_Call
fputs			move.l	#N_fputs,D0
				bra.s	Handle_pREPc_Call
fread			move.l	#N_fread,D0
				bra.s	Handle_pREPc_Call
fseek			move.l	#N_fseek,D0
				bra.s	Handle_pREPc_Call
fwrite			move.l	#N_fwrite,D0
				bra.s	Handle_pREPc_Call
printf			move.l	#N_printf,D0
				bra.s	Handle_pREPc_Call
scanf			move.l	#N_scanf,D0
				bra.s	Handle_pREPc_Call
sscanf			move.l	#N_sscanf,D0
				bra.s	Handle_pREPc_Call
sprintf			move.l	#N_sprintf,D0
				bra.s	Handle_pREPc_Call
gets 			move.l	#N_gets,D0
				bra.s	Handle_pREPc_Call
f_addr 			move.l	#N_f_addr,D0
				bra.s	Handle_pREPc_Call
.filbuf			move.l	#N_filbuf,D0
				bra.s	Handle_pREPc_Call
.flsbuf			move.l	#N_flsbuf,D0
				bra.s	Handle_pREPc_Call
strcpy			move.l	#N_strcpy,D0
				bra.s	Handle_pREPc_Call
strcmp			move.l	#N_strcmp,D0
				bra.s	Handle_pREPc_Call
strlen			move.l	#N_strlen,D0
				bra.s	Handle_pREPc_Call
puts			move.l	#N_puts,D0
				bra.s	Handle_pREPc_Call
strncat			move.l	#N_strncat,D0
				bra.s	Handle_pREPc_Call
strncmp			move.l	#N_strncmp,D0
				bra.s	Handle_pREPc_Call
strncpy			move.l	#N_strncpy,D0
				bra.s	Handle_pREPc_Call
strrchr			move.l	#N_strrchr,D0
*				bra.s	Handle_pREPc_Call		* --> last branch not needed

*****************************************************************************

pREPcTablePointer	equ		(TableBase+(pREPcTableOffset*OffsetSize))

Handle_pREPc_Call:
	ifne	PosIndCI
				lea		DCIheader,A0		*get pointer to header
	endc
	ifeq	PosIndCI
				lea		DCIheader(A5),A0	*get pointer to header (PI code)
	endc
				adda.l	#pREPcTablePointer,A0
				move.l	(A0),A0				*get pointer to Jump Table
				move.l	0(A0,D0.W),A0		*get pointer to function
				jmp		(A0)				*jump to function




***************************************************************************
*						NI System Calls
***************************************************************************

N_SignalOut				equ	0*OffsetSize
N_ControlOut			equ	1*OffsetSize
N_VXIregBase			equ	2*OffsetSize
N_CreateBuf				equ	3*OffsetSize
N_DuplicateCommonBuf	equ	4*OffsetSize
N_DCIParserEnq			equ	5*OffsetSize
N_DCIParserDeq			equ	6*OffsetSize
N_DCIParserQuery		equ	7*OffsetSize
N_SafeReadWord			equ	8*OffsetSize
N_SafeWriteWord			equ	9*OffsetSize
N_SafeReadByte			equ	10*OffsetSize
N_SafeWriteByte			equ	11*OffsetSize
N_NVconf				equ	12*OffsetSize
N_DCIwsClear			equ 13*OffsetSize
N_HWwsClear				equ	14*OffsetSize
N_SendDORrequest		equ	15*OffsetSize
N_Enable68881			equ	16*OffsetSize
N_Disable68881			equ	17*OffsetSize
N_ClearParser			equ	18*OffsetSize
N_CIAbortNormalOperation	equ	19*OffsetSize
N_CIEndNormalOperation	equ	20*OffsetSize
N_CIReleaseDevice		equ	21*OffsetSize
N_DMAmove				equ	22*OffsetSize
N_SetTrigSrc			equ 23*OffsetSize
N_SetTrigOutFP			equ 24*OffsetSize
N_SourceTrig			equ 25*OffsetSize

						xdef	SignalOut
						xdef	ControlOut
						xdef	VXIregBase
						xdef	CreateBuf
						xdef	DuplicateCommonBuf
						xdef	DCIParserEnq
						xdef	DCIParserDeq
						xdef	DCIParserQuery
						xdef	SafeReadWord
						xdef	SafeWriteWord
						xdef	SafeReadByte
						xdef	SafeWriteByte
						xdef	NVconf
						xdef	DCIwsClear
						xdef	HWwsClear
						xdef	SendDORrequest
						xdef	Enable68881
						xdef	Disable68881
						xdef	ClearParser
						xdef	CIAbortNormalOperation
						xdef	CIEndNormalOperation
						xdef	CIReleaseDevice
						xdef	DMAmove
						xdef	SetTrigSrc
						xdef	SetTrigOutFP
						xdef	SourceTrig


SignalOut				move.l	#N_SignalOut,D0
						bra.s	Handle_NI_Call
ControlOut				move.l	#N_ControlOut,D0
						bra.s	Handle_NI_Call
VXIregBase				move.l	#N_VXIregBase,D0
						bra.s	Handle_NI_Call
CreateBuf				move.l	#N_CreateBuf,D0
						bra.s	Handle_NI_Call
DuplicateCommonBuf		move.l	#N_DuplicateCommonBuf,D0
						bra.s	Handle_NI_Call
DCIParserEnq			move.l	#N_DCIParserEnq,D0
						bra.s	Handle_NI_Call
DCIParserDeq			move.l	#N_DCIParserDeq,D0
						bra.s	Handle_NI_Call
DCIParserQuery			move.l	#N_DCIParserQuery,D0
						bra.s	Handle_NI_Call
SafeReadWord			move.l	#N_SafeReadWord,D0
						bra.s	Handle_NI_Call
SafeWriteWord			move.l	#N_SafeWriteWord,D0
						bra.s	Handle_NI_Call
SafeReadByte			move.l	#N_SafeReadByte,D0
						bra.s	Handle_NI_Call
SafeWriteByte			move.l	#N_SafeWriteByte,D0
						bra.s	Handle_NI_Call
NVconf					move.l	#N_NVconf,D0
						bra.s	Handle_NI_Call
DCIwsClear				move.l	#N_DCIwsClear,D0
						bra.s	Handle_NI_Call
HWwsClear				move.l	#N_HWwsClear,D0
						bra.s	Handle_NI_Call
SendDORrequest			move.l	#N_SendDORrequest,D0
						bra.s	Handle_NI_Call
Enable68881				move.l	#N_Enable68881,D0
						bra.s	Handle_NI_Call
Disable68881			move.l	#N_Disable68881,D0
						bra.s	Handle_NI_Call
ClearParser				move.l	#N_ClearParser,D0	
						bra.s	Handle_NI_Call
CIAbortNormalOperation	move.l  #N_CIAbortNormalOperation,D0
						bra.s	Handle_NI_Call
CIEndNormalOperation	move.l  #N_CIEndNormalOperation,D0
						bra.s	Handle_NI_Call
CIReleaseDevice			move.l  #N_CIReleaseDevice,D0
						bra.s	Handle_NI_Call
DMAmove					move.l  #N_DMAmove,D0
						bra.s	Handle_NI_Call
SetTrigSrc				move.l  #N_SetTrigSrc,D0
						bra.s	Handle_NI_Call
SetTrigOutFP			move.l  #N_SetTrigOutFP,D0
						bra.s	Handle_NI_Call
SourceTrig				move.l  #N_SourceTrig,D0

*						bra.s	Handle_NI_Call			* Last branch not needed


*****************************************************************************

NITablePointer	equ		(TableBase+(NITableOffset*OffsetSize))

Handle_NI_Call:
	ifne	PosIndCI
				lea		DCIheader,A0		*get pointer to header
	endc
	ifeq	PosIndCI
				lea		DCIheader(A5),A0	*get pointer to header (PI code)
	endc
				adda.l	#NITablePointer,A0
				move.l	(A0),A0				*get pointer to Jump Table
				move.l	0(A0,D0.W),A0		*get pointer to function
				jmp		(A0)				*jump to function


***************************************************************************



N_AbortNormalOperation	equ 0*OffsetSize
N_EndNormalOperation	equ 1*OffsetSize
N_BeginNormalOperation	equ 2*OffsetSize
N_PrintProt				equ 3*OffsetSize
N_FillHandlerToLine		equ 4*OffsetSize
N_SetIntHandlers		equ 5*OffsetSize
N_ReserveHandler		equ 6*OffsetSize
N_ReenableHandler		equ 7*OffsetSize
N_InitTIC				equ 8*OffsetSize
N_SrcTrig				equ 9*OffsetSize
N_EnableTrigSense		equ 10*OffsetSize
N_DisableTrigSense		equ 11*OffsetSize
N_WaitForTrig			equ 12*OffsetSize
N_SetTrigHandler		equ 13*OffsetSize
N_GetTrigHandler		equ 14*OffsetSize
N_AcknowledgeTrig		equ 15*OffsetSize
N_MapTrigToTrig			equ 16*OffsetSize
N_UnMapTrigToTrig		equ 17*OffsetSize
N_TrigAssertConfig		equ 18*OffsetSize
N_TrigExtConfig			equ 19*OffsetSize
N_TrigCntrConfig		equ 20*OffsetSize
N_TrigTickConfig		equ 21*OffsetSize

						xdef	AbortNormalOperation
						xdef	EndNormalOperation
						xdef	BeginNormalOperation
						xdef	PrintProt
						xdef	FillHandlerToLine
						xdef	SetIntHandlers
						xdef	ReserveHandler
						xdef	ReenableHandler
						xdef	InitTIC
						xdef	SrcTrig
						xdef	EnableTrigSense
						xdef	DisableTrigSense
						xdef	WaitForTrig
						xdef	SetTrigHandler
						xdef	GetTrigHandler
						xdef	AcknowledgeTrig
						xdef	MapTrigToTrig
						xdef	UnMapTrigToTrig
						xdef	TrigAssertConfig
						xdef	TrigExtConfig
						xdef	TrigCntrConfig
						xdef	TrigTickConfig

AbortNormalOperation	move.l	#N_AbortNormalOperation,D0
						bra.s	Handle_NI2_Call
EndNormalOperation		move.l	#N_EndNormalOperation,D0
						bra.s	Handle_NI2_Call
BeginNormalOperation	move.l	#N_BeginNormalOperation,D0
						bra.s	Handle_NI2_Call
PrintProt				move.l	#N_PrintProt,D0
						bra.s	Handle_NI2_Call
FillHandlerToLine		move.l	#N_FillHandlerToLine,D0
						bra.s	Handle_NI2_Call
SetIntHandlers			move.l	#N_SetIntHandlers,D0
						bra.s	Handle_NI2_Call
ReserveHandler			move.l	#N_ReserveHandler,D0
						bra.s	Handle_NI2_Call
ReenableHandler			move.l	#N_ReenableHandler,D0
						bra.s	Handle_NI2_Call
InitTIC					move.l  #N_InitTIC,D0
						bra.s	Handle_NI2_Call
SrcTrig					move.l  #N_SrcTrig,D0
						bra.s	Handle_NI2_Call
EnableTrigSense			move.l  #N_EnableTrigSense,D0
						bra.s	Handle_NI2_Call
DisableTrigSense		move.l  #N_DisableTrigSense,D0
						bra.s	Handle_NI2_Call
WaitForTrig				move.l  #N_WaitForTrig,D0
						bra.s	Handle_NI2_Call
SetTrigHandler			move.l  #N_SetTrigHandler,D0
						bra.s	Handle_NI2_Call
GetTrigHandler			move.l  #N_GetTrigHandler,D0
						bra.s	Handle_NI2_Call
AcknowledgeTrig			move.l  #N_AcknowledgeTrig,D0
						bra.s	Handle_NI2_Call
MapTrigToTrig			move.l  #N_MapTrigToTrig,D0
						bra.s	Handle_NI2_Call
UnMapTrigToTrig			move.l  #N_UnMapTrigToTrig,D0
						bra.s	Handle_NI2_Call
TrigAssertConfig		move.l  #N_TrigAssertConfig,D0
						bra.s	Handle_NI2_Call
TrigExtConfig			move.l  #N_TrigExtConfig,D0
						bra.s	Handle_NI2_Call
TrigCntrConfig			move.l  #N_TrigCntrConfig,D0
						bra.s	Handle_NI2_Call
TrigTickConfig			move.l  #N_TrigTickConfig,D0
*						bra.s	Handle_NI2_Call			* Last branch not needed


*****************************************************************************

NI2TablePointer	equ		(TableBase+(NI2TableOffset*OffsetSize))

Handle_NI2_Call:
	ifne	PosIndCI
				lea		DCIheader,A0		*get pointer to header
	endc
	ifeq	PosIndCI
				lea		DCIheader(A5),A0	*get pointer to header (PI code)
	endc
				adda.l	#NI2TablePointer,A0
				move.l	(A0),A0				*get pointer to Jump Table
				move.l	0(A0,D0.W),A0		*get pointer to function
				jmp		(A0)				*jump to function


***************************************************************************


				end


