
/*****************************************************************************/
/*                                                                           */
/*                	National Instruments GPIB-VXI Project                    */
/*						DownLoable Code Instruments							 */
/*                                                                           */
/*                     (C) 1989  National Instruments                        */
/*                                                                           */
/*****************************************************************************/
/*
 * PROGRAMMERS WARNING ABOUT GLOBALS:
 *	If you wish to eventually EPROM a Code Instrument, NEVER use a -global-
 * variable declaration of the type 'type var = value;' (i.e. int x = 0;).
 * If you declare a variable of this type it will get put with the initialized
 * variables IN EPROM not in RAM.  The first write to the variable will result 
 * in a BUS ERROR.  Instead of this, declare it as an unitialized variable 
 * and initialize it HERE in the PRE-INIT routine.  In that way it will be 
 * placed in RAM and yet you will still have an initial value for it before it
 * is used in the Worker or Asynch processes.
 */
/*****************************************************************************/

#include "stdio.h"
#include "DataSize.h"
#include "ResManIn.h"
#include "CommonBu.h"
#include "InclWS.h"
#include "DciStruc.h"
#include "DciFuncs.h"


extern int8 BNOtype;
extern int8 NormalOperation;


uint16 *VXIregs; 		/* Global pointer to Software Version of VXI registers */
int8 SoftReset;			/* flag for whether we are cold booting or re-booting */

/*************** WARNING ***************/
/* Do NOT call any pREPc routines from this function (printf,etc)
 * It will cause a Bus Error to occur.
 */
/*************** WARNING ***************/

/*
 *  The purpose of the pre-INIT routine is to allow the user to keep an
 * uninitialized CI header space and manually fill in the values that
 * are desired.  NOTE: This way of initing the CI header space is a requirement
 * if the CI is to be EPROMed (since the header is in RAM and will not be valid
 * when the GPIB-VXI is re-booted).  This routine should not spawn off any 
 * processes or create any exchanges; patching the CI Header area should 
 * be the main function done here.  You may also scan the Resource Manager 
 * table directly (do NOT use the local command set parser) to see if devices 
 * you require exist in this system.  Any other functions needed should be 
 * done at the beginning of the Worker or Asynch processes.
 */

uint32 DCIpreInit()
{
	extern void AsynchProcess(), WorkerProcess();


		/* Get situation booting under (restart or cold boot)  */
	SoftReset = DCIheader.SoftReset;

	if (!SoftReset)
	{		/* When the Logical Address of the CI is 0xff when this routine
			 * is entered, the CI has the option of changing it to a value it
			 * desires (self-assign) or let the GPIB-VXI runtime system assign
			 * it a Logical Address (dynamically configure).  This example shows
			 * how to self-assign the CI Logical Address.
			 */
		if (DCIheader.DCILogicalAddress == 0xff)
		{	DCIheader.DCILogicalAddress = 0x50;
		}
	}

		/* poke in address of Asynch and Worker processes */
	DCIheader.AsynchStart = AsynchProcess;
	DCIheader.WorkerStart = WorkerProcess;

	/************************************/
	/*		Setup VXI Registers			*/
	/************************************/
		/* get base address of VXI registers */
	VXIregs = &DCIheader.VXIregs[0];
	
		/* set VXI Configuration registers */
	VXIregs[IdReg] 			= DCI_ManufacturerId | DCI_AddressSpace | DCI_DeviceClass;
	VXIregs[DeviceTypeReg]  = DCI_ModelCode | DCI_RequiredMemory;

		/* NOT Passed/NOT Ready until done with Initing when the Worker process is up */
	VXIregs[StatusReg]		= 0;

		/* supports CMDR, Signal, Master and nothing else */
	VXIregs[ProtocolReg]	= (0xffff & (Protocol_INTER))  & 
							~(Protocol_CMDR | Protocol_SIGNAL | Protocol_MASTER); 

		/* no Error, not in FHS, not Locked, and NOT (DIR,DOR,ReadReady,WriteReady) */
	VXIregs[ResponseReg]	= Response_ERR | Response_FHS | Response_LOCKED;
	/************************/

		/* Initialize 488.2/Word Serial status byte */
	DCIheader.StatusByte = 0x00;


	/**********************************************/
	/* Do any other pre-initialazation steps here */
	/**********************************************/

	/*
	 ********   IMPORTANT NOTE:    *****************************************************
	 *	Remember when pre-initializing any variables here that this routine could be
	 * being called after a Soft Reset.  Any changed values that reflect configuration
	 * should be left as is.  Any memory allocated dynamically out of pSOS dynamic
	 * memory will be lost so any configuration information should be held in the
	 * Code Instruments global variable memory after the header and not changed after
	 * a Soft Reset.  Not following these rules will cause a Violation of the VXI
	 * Specification.
	 ***********************************************************************************
	*/

		/* Have not yet received Begin Normal Operation and are in CONFIG state */
	BNOtype = 0;	
	NormalOperation = 0;


#ifdef TAKE_OVER_PHYSICAL_WORD_SERIAL
	{
	extern int8 OurCommander;
	extern uint16 OurStatusByte, ProtocolError;

	OurCommander = DCIheader.CMDRtable[DCIheader.GPIB_VXI_LA];
	OurStatusByte = 0;
	DCIheader.GPIB_VXI_EventEnables[0] = 0xffffffff;
	DCIheader.GPIB_VXI_EventEnables[1] = 0xffffffff;
	DCIheader.GPIB_VXI_EventEnables[2] = 0xffffffff;
	DCIheader.GPIB_VXI_EventEnables[3] = 0xffffffff;
	}
#endif


		/* We wish to unconditionally startup this CI so return 1 .... 
		 * If a 0 is returned, the CI bootstrap process will be aborted.  This is usefull
		 * when you wish to only install CIs when there is appropriate hardware to 
		 * for it to access.
		 */
	return 1L;
}


