
/*****************************************************************************/
/*                                                                           */
/*                	National Instruments GPIB-VXI Project                    */
/*						DownLoable Code Instruments							 */
/*                                                                           */
/*                     (C) 1989  National Instruments                        */
/*                                                                           */
/*****************************************************************************/

#include "stdio.h"
#include "DataSize.h"
#include "ResManIn.h"
#include "S_EVENT.h"
#include "CommonBu.h"
#include "InclWS.h"
#include "DciStruc.h"
#include "DciFuncs.h"
#include "UserDMA.h"


	/* Quick reference variables */
la_entry *rmtable;
int8 MyLA, Cmdr, GPIB_VXI_LA;
uint16 ParserId;
int8 SecondaryAddress;
extern uint16 *VXIregs;
extern int8 SoftReset;
extern uint32 CIEventEnables[4];

	/* Keeps track of number of Word Serial Data In Events */
extern int32 WS_DATA_IN_NUM;
extern int8 GetOutput;


/*
 *  The purpose of the Worker process is to concentrate on the device
 * specific operation of the DCI.  The Asynch process handles most of the
 * VXI protocols and notifies other processes (such as the Worker process)
 * only in the case of the event affecting its operation.
 *	The Worker process is meant to simulate a VXI Message Based device's
 * power-on and runtime activities.  It has time to set its PASSED bit in
 * its STATUS register and MUST wait for Begin Normal Operation to occur 
 * (the Asynch process will notify it of that fact).  It may then go about
 * performing any I/O and other operations it sees fit to execute within
 * the bounds of the VXI Specification.  Basically, the worker process can
 * do just about anything.... It can take buffers in off the GPIB, put buffers
 * on the GPIB, create a local command set of its own, access Register based
 * devices, access Message Based devices, access foreign devices, configure
 * and re-configure the VXI system, etc.
 */

void WorkerProcess()
{
	uint16 caught;

	
	start("0.0","0.0","0.0"); 			/* Must call to use UART (Serial Port) I/O */

		/* Initialize global variables if cold booting */
	if (!SoftReset)
		InitGlobals();

		/* If you wish to use the 68881 math co-processor (any floating point
		 * operations) then you MUST call 'Enable68881()' once at the beginning
		 * of EACH process that will use it.  No floating point operations may
		 * be performed in any function prior to calling the enable function.  
		 * This includes even defining float variables in the root function.
		 * If no 68881 is present and floating point operations wish to be performed
		 * then a math library must be supplied.  'printf' and 'scanf' assume
		 * an IEEE format floating point value.
		 */
#ifdef IF_HAVE_AND_NEED_68881
	Enable68881();				/* returns 1 if 68881 present, 0 if not */
#endif


	/************************************/
	/* 	Perform any self-test or other operations needed before beginning
	 * operation HERE.  Then set your PASSED bit.
	 */
	/************************************/


		/* Set Passed bit and Not Ready bit */
	VXIregs[StatusReg] |= Status_PASSED;

		/* NOTE ----> do not Call GPIB-VXI Local Parser until BNO is received */
		/* Wait for a Begin Normal Operation to occur */
	wait_v(S_EVENT_BNO,1,0,&caught);


	/*******************************************************/
	/* DO ANY POST Begin Normal Operation preparation HERE */
	/*******************************************************/

		/* If cold booting set Secondary Address (otherwise is already set) */
	if (!SoftReset)
		SetSecondaryAddress();


#ifdef USE_PHYSICAL_WORD_SERIAL
	{
	extern PhysicalWSread();
	uint32 PWSpid;

	TakeOverWS();
    spawn_p("CIWS",0,100,0x100,0x100,0,&PWSpid);
    activate_p(PWSpid,PhysicalWSread);
	}
#endif

#ifdef ALL_INTERRUPT_HANDLERS_ARE_RORA
	DCIheader.RORAhandler = 0xffff;
#endif

	MainLoop();
	suspend_p(0);
}




MainLoop()
{
	int8 ret, rt, endflag;
	uint16 stat,caught;
	CommonBuf *WSBuf, *Outbuf;
	WSbuf WSb;
	WScom WSc;
	extern CommonBuf *Read852();

		/* Initialize CDS 852 Global Variables and reset the 852 module */
	Init852();

	printf("\r\nNational Instrument's CDS 73A-852 Code Instrument Adaptor Running\r\n");

	for(;;)
	{ 		
			/* if enabled for DIR response interrupts send one */
		if (!(DCIheader.RespEna & RESP_DIR))		/* active low */
			SignalOut(Cmdr, (((RESP_DIR | RESP_FHS | RESP_ERR)<<8) | MyLA));

		DCIheader.StatusByte = 0xff03;

			/* wait for a Word Serial Buffer In */
		wait_v(S_EVENT_DCI_IO,1,0,&caught);

		while (WS_DATA_IN_NUM > 0)
		{
			DCIheader.StatusByte = 0xff83;

			WSb.mode = 0;
			WSb.MaxReadCount = 0x200;
			WSb.Wait = 0L; 		         /* wait forever for buffer allocation */
				/* Read in the actual buffer */
			d_read(DEV_WSsl,MyLA,&WSb,&WSBuf,0);
			if (!(WSb.RetVal & WSb_RET_IODONE))
			{	printf("D_READ ERROR returns 0x%x \r\n",WSb.RetVal);
				continue;
			}

			if (WSBuf->data[0] == '!' && WSBuf->data[1] == '!')		/* local command? */
				ExecuteLocalCommand(WSBuf);
			else
				Write852(WSBuf);

				/* MUST FREE BUFFER in the DCI */
			rt = free_seg(WSBuf);
			if (rt != 0x00)
				printf("Freeing error for WSbuf (Inputted buffer).\r\n");

				/* Finished processing a buffer so decrement count (Make sure END)*/
			WS_DATA_IN_NUM--;
		}

		if (GetOutput)
		{	
			DCIheader.StatusByte = 0xff23;

			WSBuf = Read852();
	
				/* if a buffer returned send it out */
			if (WSBuf)
			{ 	WSb.mode = 0;
				d_write(DEV_WSsl,MyLA,&WSb,&WSBuf,0);

					/* if error sending buffer, throw it away */
				if (!(WSb.RetVal & WSb_RET_IODONE))
				{	/*printf("D_WRITE ERROR returns 0x%x \r\n",WSb.RetVal);*/
					free_seg(WSBuf);
					continue;
				}
			}

			GetOutput = 0;
		}
	
	}
}




InitGlobals()
{
		/* Get My Logical Address, Commanders and the GPIB-VXI's (if needed later) */
	MyLA 		= DCIheader.DCILogicalAddress; 
	GPIB_VXI_LA = DCIheader.GPIB_VXI_LA;
	Cmdr 		= DCIheader.CmdrLogicalAddress;

	ParserId	= DCIheader.ParserId;
	rmtable		= &DCIheader.la_table[0];

	if (!SoftReset)
	{		/* active low:  globally enabled for reponses/events */
		DCIheader.AsynMode  =  0xffff & ~(ASYN_RESP_ENA | ASYN_EVENT_ENA);
		rmtable[MyLA].AsynMode   =  0xffff & ~(ASYN_RESP_ENA | ASYN_EVENT_ENA);
			/* active low: not enabled for any responses (until commander tells us) */
		DCIheader.RespEna   = 0xffff;
		rmtable[MyLA].RespENA    = 0xffff;
			/* enable all bit-wise event enable masks */
		CIEventEnables[0] = 0xffffffff;
		CIEventEnables[1] = 0xffffffff;
		CIEventEnables[2] = 0xffffffff;
		CIEventEnables[3] = 0xffffffff;
	}
		/* no Secondary Address set yet */
	SecondaryAddress = 255;
}




/*	
 *	This function sends a command to the GPIB-VXI Local Command Set in order
 * to set this CI's Secondary Address.  It tries to set the Secondary Address
 * to the upper five bits of the Logical Address but if it can't it finds the
 * next available Secondary Address (if any).
 */

SetSecondaryAddress()
{
	int8 Pcom[30], SecAddr;
	CommonBuf *p;

		/* Secondary address is upper 5 bits of Logical address */
	SecAddr = (MyLA >> 3);

	for (;;)
	{
			/* Create Command string for GPIB-VXI Local Command Set Parser */
		sprintf(Pcom,"lasaddr %d,%d", MyLA, SecAddr);

			/* Since 'LaSaddr' has no response, response will occur only if error */
		if ((p = DCIParserQuery(ParserId, Pcom)) != 0)
		{		
			free_seg(p); 					/* free up error return buffer */
			SecAddr++;						/* try next Secodary Address */
			if (SecAddr == (MyLA >> 3))		/* if got back to first one, give up */
				return 0;
			if (SecAddr > 0x1e)				/* wrap around */
				SecAddr = 0;
		}
		else
			break;							/* Secondary Address set, so leave */
	}

		/* copy new Secondary Address to global variable */
	SecondaryAddress = SecAddr;

	return 1;
}



#ifdef EXAMPLES_TO_USE_WHEN_NEEDED

/*
 * The following function is used for a CI to commit suicide (so to speak).  This
 * function may be useful for Code Instruments that wish to exist only under
 * certain circumstances (such as a particular piece of hardware being available).
 */
DeleteMyself()
{
	int8 Pcom[30];
	CommonBuf *p;

		/* Create Command string for Local Parser */
	sprintf(Pcom,"cidelete %d", MyLA);

		/*
		 * Note that if deletion was done, no more further lines of code
		 * will be executed.  The odds are that even if an error did occur,
		 * the CI will be unable to continue executing.  This is because it
		 * will be marked as invalid and have most of its resources taken
		 * away from it (depending on where the error occurred).
		 */

		/* Since 'LaSaddr' has no response, response will occur only if error */
	if ((p = DCIParserQuery(ParserId, Pcom)) != 0)
	{	printf("DCI ERROR, unable to Delete myself Error = %s\r\n",p->data);
		free_seg(p);
		return 0;
	}
	return 1;
}





/*
 * 	The following are examples of how to send other commands to the Local
 * Command Set.
 */

CommandsExample()
{
	CommonBuf *p;
	int8 *q, gotone;
	uint16 caught;

		q = "rmentry? 0"; 		printf("Command  = [%s] \r\n",q);
		p = DCIParserQuery(ParserId, q);
		while (p != 0)
		{	printf("Response: %s \r\n", p->data);
			free_seg(p);
			p = DCIParserDeq(ParserId);
		}
		printf("DONE: No more response given!\r\n");


		q = "consmode 1"; 		printf("Command  = [%s] \r\n",q);
		p = DCIParserQuery(ParserId, q);
		while (p != 0)
		{	printf("Response: %s \r\n", p->data);
			free_seg(p);
			p = DCIParserDeq(ParserId);
		}
		printf("DONE: No more response given!\r\n");


		q = "dpram?"; 		printf("Command  = [%s] \r\n",q);
		p = DCIParserQuery(ParserId, q);
		while (p != 0)
		{	printf("Response: %s \r\n", p->data);
			free_seg(p);
			p = DCIParserDeq(ParserId);
		}
		printf("DONE: No more response given!\r\n");


		q = "allhandlers?"; 		printf("Command  = [%s] \r\n",q);
		p = DCIParserQuery(ParserId, q);
		while (p != 0)
		{	printf("Response: %s \r\n", p->data);
			free_seg(p);
			p = DCIParserDeq(ParserId);
		}
		printf("DONE: No more response given!\r\n");


		q = "consmode 0"; 		printf("Command  = [%s] \r\n",q);
		p = DCIParserQuery(ParserId, q);
		while (p != 0)
		{	printf("Response: %s \r\n", p->data);
			free_seg(p);
			p = DCIParserDeq(ParserId);
		}
		printf("DONE: No more response given!\r\n");

}




/*
 *	The following is are examples of how to Send Word Serial Commands to
 * another Message Based device (DCI or physical).  Remember: Commands
 * should be sent only to YOUR Message Based SERVANTS.
 */

SendWScommandExamples()
{
	WScom WSc;
	int8 la;
	
printf("Sending Read STB...\r\n");
	la = 16;
	WSc.Command = WS_READ_STB;
	WSc.mode = WS_MODE_RESPONSE;
	d_ctrl(DEV_WS,la,WS_CTRL_Command,&WSc);
printf("DONE...Status = 0x%x   (ret = 0x%x) \r\n",WSc.Response,WSc.RetVal);


printf("Sending Device Clear...\r\n");
	la = 16;
	WSc.Command = WS_CLEAR;
	WSc.mode = 0;
	d_ctrl(DEV_WS,la,WS_CTRL_Command,&WSc);
printf("DONE... (ret = 0x%x) \r\n",WSc.RetVal);

}




/*
 * 	The following are examples of how to get Non-Volatile configuration
 * information for the DCI.  There are 32 longword values (0-31) that can
 * be retrieved.  To set the value of the configuration, the user must
 * use the Nonvolatile Configuration Menu on boot-up (or from pROBE command
 * 'CONF').
 */

NVconfExample()
{
	uint32 ret, confinfo;

	ret = NVconf(0, &confinfo);
	ret = NVconf(1, &confinfo);
	ret = NVconf(31, &confinfo);
}





UseMemoryToMemoryDMAexample()
{
	uint32 ret;

		/* transfer 32K bytes (in 16 bit DMA transfers) from 0x80000 to 0xf0000 */
	ret =  DMAmove((uint32)0x80000, (uint32)0xf0000, (uint32)0x8000, 0);
	if (ret != 0)
		printf("DMA transfer errored\r\n");

		/* transfer 0x1000 bytes from 0x90000 to 0x210000 a 8 bits at a time */
	ret = DMAmove((uint32)0x90000, (uint32)0x210000, (uint32)0x1000,
					DMA_MODE_8bitDev | DMA_MODE_8bitOper));
	if (ret != 0)
		printf("DMA transfer errored\r\n");

}





/*
 *	To have the GreenHills C compiler generate math floating point
 * instructions for the 68881, you must you the compiler flags:
 *			-X22 -X99 -X129
 */
test68881()
{
	double a,b,c;
	uint32 i;

	printf("\r\n");
	printf("Enter test for 68881 \r\n");

	a = 12345.6789;
	printf("this is a float: %20.10e \r\n", a);

    a = 30.0;
    b = 30.0;
    c = a*b;
	printf("a = %12.5e, b = %12.5e, c = %12.5e \r\n",a,b,c);
    a = (b*8.0+39.0) / c;
	printf("a = %12.5e, b = %12.5e, c = %12.5e \r\n",a,b,c);
    b = c + a - 8.0;
	printf("a = %12.5e, b = %12.5e, c = %12.5e \r\n",a,b,c);

	printf("\r\n");
}




/*
 *	The following function can be used to see if a device is in the system.
 * Any or all of the following characteristics may be used to search for the
 * device with:
 *				---- Field ----						---- Range ----
 *		Manufacturer Identification number		0-FFF		(FFFF = do not use)
 *		Model Code								0-FFF		(FFFF = do not use)
 *		Device Class							0-3			(FF = do not use)
 *		Commander's Logical Address				0-FE		(FF = do not use)
 *
 *	The logical address of the first device with the characteristics provided
 * will be returned.  If no device with the characteristics given is found
 * then 0xff is returned.  The function also returns a boolean (0 or 1) if the
 * device was found.
 *
 * example:
 *		FindDevLA(0xff6, -1, 3, -1, &retla);
 *
 *	This would attempt to find a device made by National Instruments
 * with any model code, Register based, and with any commander.
 */

FindDevLA(ManId,ModelCode,DevClass,CmdrLa, ReturnLa)
	uint16 ManId, ModelCode;
	uint8  DevClass, CmdrLa;
	uint8  *ReturnLa;
{
	int8 		la, found;
	int8		*hierarchy, *laValid, *laDCI, GPla;


		/* to help with speed get local pointers to device tables */
	hierarchy 	= &DCIheader.CMDRtable[0];		/* hierarchy table (list of cmdrs for LAs */
	laValid 	= &DCIheader.laValid[0];		/* boolean array of physical devices */
	laDCI 		= &DCIheader.laDCI[0];			/* boolean array of Code Instruments */
	GPla 		= DCIheader.GPIB_VXI_LA;		/* GPIB-VXI's Logical Address */

	found = 0;
	for (la = 0; la <= 0xfe; la++)
	{	
			/* Is this table entry valid? (physical device, CI, or GPIB-VXI?) */
		if (!laValid[la] && !laDCI[la] && (la != GPla))
			continue;

			/* test Manufacturer Id */
		if (ManId != 0xffff)
		{	if (rmtable[la].ManId != ManId)
				continue;
		}

			/* test Model Code */
		if (ModelCode != 0xffff)
		{	if (rmtable[la].ModelCode != ModelCode)
				continue;
		}

			/* test Device Class */
		if (DevClass != 0xff)
		{	if (rmtable[la].DevClass != DevClass)
				continue;
		}

			/* test Commander's Logical Address */
		if (CmdrLa != 0xff)
		{	if (hierarchy[la] != CmdrLa)
				continue;
		}

		found = 1;
		break;
	}

	if (found)
		*ReturnLa = la;
	else
		*ReturnLa = 0xff;
		
	return found;
}

#endif


