
/*****************************************************************************/
/*                                                                           */
/*                	National Instruments GPIB-VXI Project                    */
/*						DownLoable Code Instruments							 */
/*                                                                           */
/*                     (c) 1989  National Instruments                        */
/*                                                                           */
/*****************************************************************************/
/*
 *	The following defines the 'CommonBuf'.  This is a C typedef that is used
 * for ALL I/O communication on the GPIB-VXI (GPIB, Word Serial, Parser, DCI's,
 * and Shared Memory).  This buffer type can be transferred from one source to
 * any other source without modification.  Only the proper driver call is
 * required.
 *	'CommonBuf' is a variable length buffer.  The function 'CreateBuf' creates
 * one of these buffers with a data field of N * 256 bytes out of pSOS dynamic
 * memory.  The field 'MaxSize' tells what size data field was allocated at 
 * creation time.  With this ability, the user can create data buffers from 0 
 * bytes to 16 MegaBytes.  Only the amount of pSOS dynamic memory availible
 * is the restriction.
 *
 * WARNING:  Never access fields within the CommonBuf as offsets from the base.
 * Only access the NAMES of the fields.  With the pending changes to the 
 * Shared Memory Protocol, the defintion of the CommonBuf WILL CHANGE (but the
 * names will be the same).
 */


	/* Used to satisfy C structure definition only */
#define CommonBufMaxSIZE		(4096 << 10)			/* 4 MegaBytes */


typedef struct CommonBuf{
	uint32	useCount;					/* NI reserved -- link in Shared Memory */
	uint32	byteCount;					/* total bytes in this buffer 			*/
	uint16	Source;						/* used only in Shared Memory 			*/
	uint16	Destination;				/* used only in Shared Memory 			*/
	int8	type;						/* Data, Trigger, Clear 	   			*/
	int8	EndFlag;					/* 0= No End, 1=End	with buffer			*/
	uint16	MaxSize;					/* Allocated size of data in 256 byte chunks */
	int8   data[CommonBufMaxSIZE];		/* data up to 'MaxSize' * 256  bytes */
} CommonBuf;


extern CommonBuf *CreateBuf(), *DuplicateCommonBuf();


