
/*****************************************************************************/
/*                                                                           */
/*                National Instruments GPIB-VXI Project                      */
/*                                                                           */
/*                   (C) 1989  National Instruments                          */
/*                                                                           */
/*****************************************************************************/
/*
	This file contains definitions needed for the Word Serial Drivers
 */

	/* pSOS driver locations */
#define DEV_WS 		0x0200			/* WS commander driver major/minor address*/
#define DEV_WSsl 	0x0300			/* WS slave driver major/minor address */

	/* WORD SERIAL COMMAND VALUES */
#define WS_BYTEAV_0     0xbc00		/* Byte Available without END */
#define WS_BYTEAV_1     0xbd00		/* Byte Available with END */
#define WS_BYTEAV_END   (1<<8)		/* Byte Available END position */
#define WS_BYTEAV_MASK  0xfe00		/* Upper 8 bits minus END bit */
#define WS_BYTERQ       0xdeff		/* Byte Request command */
#define WS_BYTERQ_END   (1<<8)

#define WS_CLEAR		0xffff		/* device clear */
#define WS_TRIGGER		0xedff		/* trigger */
#define WS_READ_SERV    0xceff		/* Read Servant Area Command 			*/
#define WS_DEVGRANT     0xbf00		/* Device Grant (upper 8 bits)			*/
#define WS_REL_DEVICE   0x8e00		/* Release device (upper 8 bits)		*/
#define WS_ID_COMMANDER 0xbe00		/* Identify commander (upper 8 bits)	*/
#define WS_READ_STB     0xcfff		/* Read Status Byte						*/

#define WS_ASSIGN_HNDLR 0xa900		/* Assign Handler (VXI Interrupts)      */
#define WS_READ_HNDLRS	0xc700		/* Read Handlers (Number Int Handlers)  */
#define WS_READ_LINE	0x8c00		/* Read Handler Line					*/

#define WS_ASSIGN_INT 	0xaa00		/* Assign Interrupter (VXI Interrupts)  */
#define WS_READ_INTS	0xcaff		/* Read Interrupters (Num Interrupters) */
#define WS_READ_INTLINE	0x8d00		/* Read Interrupter Line				*/

#define WS_READ_MODID	0xccff		/* Read MODID							*/
#define WS_LOWER_MODID	0xae00		/* Set Lower MODID						*/
#define WS_UPPER_MODID	0xad00		/* Set Upper MODID						*/
#define WS_MODID_ENA	0x0080		/* Modid Enable bit for set upper/lower */

#define WS_BEGIN_NORM_A 0xfdff  	/* Begin Normal Operation--AUTO 		*/
#define WS_BEGIN_NORM   0xfcff  	/* Begin Normal Operation--no AUTO 		*/
#define WS_ENO			0xc9ff		/* End Normal Operation					*/
#define WS_ANO			0xc8ff		/* Abort Normal Operation				*/

#define WS_ASYN_MODE	0xa8ff		/* Asynchronous Mode Control		*/
#define ASYN_RESP_ENA	0x0008		/* 0 = response ints enabled		*/
#define ASYN_EVENT_ENA	0x0004		/* 0 = event ints enabled			*/
#define ASYN_RESP_INT	0x0002		/* 0 = response ints VME 1=signal	*/
#define ASYN_EVENT_INT	0x0001		/* 0 = event ints VME	 1=signal	*/

#define WS_CTRL_EVENT	0xaf00		/* Control Event (ena/dis event gen)    */
#define WS_RESP_ENA		0x8fff		/* Response interrupt enable			*/
#define RESP_DOR		0x0020		/* Enable Data Out Ready interrupt		*/
#define RESP_DIR		0x0010		/* Enable Data In Ready interrupt		*/
#define RESP_ERR		0x0008		/* Enable ERR bit interrupt				*/
#define RESP_RR			0x0004		/* Enable Read Ready interrupt			*/
#define RESP_WR			0x0002		/* Enable Write Ready interrupt			*/
#define RESP_FHS		0x0001		/* Enable Fast Handshake Active inter.	*/

#define WS_RD_PROT_ERR 	0xcdff		/* Read Protocol Error (Error query)	*/
#define PROT_E_NO_ERROR		0xffff	/* No error						*/
#define PROT_E_GENERIC		0xfffe	/* Generic error (not in Spec 1.3) */
#define PROT_E_MQE			0xfffd	/* Multiple Query Error 		*/
#define PROT_E_UNREC_COM	0xfffc	/* Unrecognized command  		*/
#define PROT_E_DIR_VIOL		0xfffb	/* DIR violation (BAV sent when not DIR) */
#define PROT_E_DOR_VIOL		0xfffa	/* DOR violation (BRQ sent when not DOR) */
#define PROT_E_RR_VIOL		0xfff9	/* RR violation (resp read when not RR) */
#define PROT_E_WR_VIOL		0xfff8	/* WR violation (Command written when not WR) */

#define WS_READ_PROT    0xdfff		/* Read Protocols 				*/
#define PROT_DEVDEP		0x87ff		/* Mask for Device dependent	*/
#define PROT_SPEC_1_2	(1<<15)		/* 0 = VXI Spec 1.2 device 		*/
#define PROT_RG			(1<<9)		/* 0 = Response Generation		*/
#define PROT_EG			(1<<8)		/* 0 = Event Generation			*/
#define PROT_E    		(1<<7)		/* 0 = ERR bit supported		*/
#define PROT_PI    		(1<<6)		/* 0 = WS Interrupt commands 	*/
#define PROT_PH    		(1<<5)		/* 0 = WS Int Handler commands  */
#define PROT_TRG   		(1<<4)		/* 0 = WS Trigger commandd		*/
#define PROT_I4    		(1<<3)		/* 0 = 488.2 supported			*/
#define PROT_I    		(1<<2)		/* 0 = VXI Instrument prot		*/
#define PROT_ELW   		(1<<1)		/* 0 = WS Extended Long word	*/
#define PROT_LW    		(1<<0)		/* 0 = WS Long word protocol	*/



	/*******************************/
	/* Word Serial Command Sending */
	/*******************************/

	/* structure needed to send a Word Serial command */
typedef struct WScom {
	uint16 mode;			/* How should command be sent */
	uint16 RetVal;			/* return status */
	uint16 Command;			/* actual command to send */
	uint16 Response;		/* reponse (if applicable) returned */
	uint32 PID;				/* filled in internally */
} WScom;

	/* Bit definitions for 'mode' field */
#define WS_MODE_RESPONSE	(1<<0)				/* Command requires a reponse */

	/* Bit definitions for 'RetVal' field */
#define WS_RET_IODONE		(1<<0)			/* I/O successfully completed */
#define WS_RET_TIMO_SEND	(1<<1)			/* I/O timed out on SEND of command */
#define WS_RET_TIMO_REC		(1<<2)			/* I/O timed out on RESPONSE for command */
#define WS_RET_UnReqCom		(1<<3)			/* Unrecognized Command event received */
#define WS_RET_LA_DNE		(1<<4)			/* Logical Address Does Not Exist */
#define WS_RET_InvalidLA	(1<<5)			/* Invalid Logical Address to talk to */
#define WS_RET_MQE			(1<<6)			/* Mulitple Query Error */
#define WS_RET_BERR			(1<<7)			/* An unrecoverable Bus Error occurred */
#define WS_RET_RDERR_NSUP	(1<<10)			/* Read Protocol Error failed (not supported or TIMO */
#define WS_RET_DIR_VIOL		(1<<11)			/* A DIR violation occurred (BAV sent when not DIR) */
#define WS_RET_DOR_VIOL		(1<<12)			/* A DOR violation occurred (BRQ sent when not DOR) */
#define WS_RET_RR_VIOL		(1<<13)			/* A response was read when not read ready */
#define WS_RET_WR_VIOL		(1<<14)			/* A command was written while not write ready */

	/* Word Serial Commander 'type' field for d_ctrl call */
#define WS_CTRL_Command			0					/* 'd_ctrl' for sending a WS command */
#define WS_CTRL_Clear			1					/* 'd_ctrl' to perform Clear abort */
#define WS_CTRL_UnRecCom		2					/* 'd_ctrl' to perform UnRecCom abort */
#define WS_CTRL_Trigger			3					/* 'd_ctrl' for sending a WS Trigger */
#define WS_CTRL_QueryResponse	4					/* 'd_ctrl' for receiving a WS response */



	/********************************/
	/* Word Serial Response Sending */
	/********************************/

	/* structure needed to send a Word Serial response */
typedef struct WSresp {
	uint16 RetVal;				/* return status */
	uint16 Response;			/* actual response to be sent */
} WSresp;


	/* Bit positions for 'RetVal' in response structure */
#define WSsl_RET_IODONE			(1<<0)		/* I/O successfully completed */
#define WSsl_RET_MQE			(1<<1)		/* Multiple Query Error (reponse still pending)*/
#define WSsl_RET_NoRespPend		(1<<2)		/* Called with no response pending */
#define WSsl_RET_InvalidCaller	(1<<3)		/* invalid calling Logical address */

	/* Word Serial response 'type' field for d_ctrl call */
#define WSsl_CTRL_Response		0				/* d_ctrl call is to perform a WS reponse */
#define WSsl_CTRL_GenProtErr	1				/* d_ctrl call is to generate a protocol error */



	/***************************************/
	/* Word Serial Buffer Sending/Receving */
	/***************************************/

	/* structure needed to send/receive a Word Serial buffer */
typedef struct WSbuf {
	uint16 mode;			/* How should command be sent */
	uint16 RetVal;			/* return status */
	uint32 MaxReadCount;	/* Size to create buffer in bytes (for READ-only if needed) */
	uint32 Wait;			/* amount of time to wait (0=forever) if need to Alloc buffer */
} WSbuf;

	/* Bit definitions for 'mode' field */
#define WSb_MODE_NO_FreeBuf		(1<<0)		/* WRITEs: do NOT free buffer when through */
#define WSb_MODE_NO_DirDorAbort	(1<<1)		/* Cmdr:do NOT abort on !DIR (writes), !DOR (reads)*/
#define WSb_MODE_NO_SendDirDor	(1<<2)		/* Srvt:do NOT send DIR or DOR reponse signal */

	/* Bit definitions for 'RetVal' field for commander driver */
#define WSb_RET_IODONE			(1<<0)		/* I/O successfully completed */
#define WSb_RET_END				(1<<1)		/* END was received on Read */
#define WSb_RET_TC				(1<<2)		/* Terminal Count--buffer was completely filled */
#define WSb_RET_DirDorAbort		(1<<3)		/* device was !DIR (writes) or !DOR (reads) */
#define WSb_RET_Clear			(1<<4)		/* Device Clear received during transfer */
#define WSb_RET_ErrALLOC		(1<<5)		/* Unable to allocate buffer */
#define WSb_RET_ErrFREE			(1<<5)		/* Unable to free buffer sent in */
#define WSb_RET_TIMEOUT			(1<<6)		/* buffer transfer timed out polling on a bit */
#define WSb_RET_MQE				(1<<7)		/* Multiple Query Error ocurred during transfer */
#define WSb_RET_InvalidLA		(1<<8)		/* Invalid Logical Address to communicate with */
#define WSb_RET_UnabletoInOut	(1<<9)		/* (DCI servant only): no data In pending (reads) */
											/*	or Out path full (writes) */
#define WSb_RET_ProtocolError	(1<<10)		/* One of the Protocol errors occurred */

	/* INTERNAL USE: 
	 * Bit definitions for 'RetVal' field for servant driver (DIFFERENT from ABOVE) */
#define wsSL_XXX11   	     	(1<<11)
#define wsSL_XXX12           	(1<<12)
#define wsSL_XXX13  	       	(1<<13)
#define wsSL_XXX14           	(1<<14)
#define wsSL_XXX15          	(1<<15)


