
/*****************************************************************************/
/*                                                                           */
/*                	National Instruments GPIB-VXI Project                    */
/*                                                                           */
/*                      (C) 1989  National Intruments                        */
/*                                                                           */
/*****************************************************************************/

/* resource manager data structures */

#define NLA			254		/* number of last logical address */
#define NSLOTS		12		/* number of physical slots	   */

#define MEMSPACE_A16	0
#define MEMSPACE_A24	1
#define MEMSPACE_A32	2

typedef struct la_entry {
	int8	*regbase;		/* location of VXI registers			*/
	uint16	slot;			/* physical slot in which it resides 	*/
	uint16	DevClass;		/* device class 0=memory, 1=extended,   */
							/*    2=message based, 3=register based */
	uint16	SubClass;		/* for Extended Class Devices 			*/
	uint16	ManId;			/* manufacturer identification number	*/
	uint16	ModelCode;		/* manufacturer model number			*/
	uint16	servants;		/* integer index into ServantList[] 	*/
	int8	servsize; 		/* Servant Size, only if a SAC device 	*/
	int8	memspace;		/* 0 = A16, 1=A24, 2=A32 				*/
	int8	*membase;		/* address of A24 or A32 address space	*/
	uint32	memsize;		/* address space size in actual bytes 	*/
	uint16	prot;			/* protcols supported (from Read Prot)	*/
	uint16	RespENA;		/* Response Interrupts Enabled			*/
	uint16	AsynMode;		/* Asynchronous Mode Control Response	*/
	uint16	SADDR;			/* secondary address currently attached */
	uint16	StatusState;	/* 3=PASSED,1=EXTENDED,0=FAILED,2=DIAG	*/
	uint16	flags;			/* various flags						*/
} la_entry;


/* Status State */
#define LA_PASSED		3
#define LA_EXTEND		1
#define LA_FAILED		0
#define LA_DIAG			2


/* flags */
#define LA_ALIVE	(1<<0)	/* 1=was alive at one time, 0=never alive */
#define LA_DEAD		(1<<1)	/* 1=was forced offline,0=currently alive */
#define LA_READ_STB	(1<<2)	/* 1= Device can respond to WS Read STB */


