
/*****************************************************************************/
/*                                                                           */
/*                 National Instruments GPIB-VXI Project                     */
/*                                                                           */
/*                    (C) 1989  National Instruments                         */
/*                                                                           */
/*****************************************************************************/
/* The following file is a sample function for a CI to take over the PHYSICAL
 * Word Serial registers of the GPIB-VXI.  This function is a guideline and
 * as such should be modified for the users application.
 */

#include "stdio.h"
#include "DataSizes.h"
#include "ResManIncl.h"
#include "InclWS.h"
#include "DciStruct.h"
#include "CommonBuf.h"
#include "DciFuncs.h"


extern int8 MyLA, GPIB_VXI_LA;
extern uint16 ParserId;

#define WSPHYSCOMMANDDEBUG 1


	/* register access macros for GPIB-VXI registers (no need for la)*/
#define GPRegBase   0xfe0000
#define gpReg(r)      ((*(uint16 *)(GPRegBase+r)))
#define gpOut(r,v)    (gpReg(r) = (v))
#define gpIn(r)       gpReg(r)

	/* Response Register */
#define Rresponse   0x0a
#define Wresponse   0x24
#define R_RESERVED  (1<<14)     /* 1 always, reserved */
#define R_DOR       (1<<13)     /* 1 = ready to output data */
#define R_DIR       (1<<12)     /* 1 = ready to input data */
#define R_ERR       (1<<11)     /* 0 = error unreported */
#define R_RREADY    (1<<10)     /* 1 = data low has data in it*/
#define R_WREADY    (1<<9)      /* 1 = ready for write to data low*/
#define R_FHS       (1<<8)      /* 0 = fast hanshake mode on */
#define R_LOCKED    (1<<7)      /* 0 = access locked to local sources*/

	/* MODID register */
#define modid       0x26
#define M_ENABLE    (1<<13)     /* 1 = enable modid drivers*/
#define M_MODID     0x1fff      /* mask for MODID lines */
#define M_MODID_00_06   0x007f  /* mask for MODID lines 00-06 */
#define M_MODID_07_12   0x1f80  /* mask for MODID lines 07-12 */
#define M_MODIDS(a) (1<<a)      /* a = slot, 1 = drive modid line/modid set*/


extern int8 NormalOperation;
extern int8 GPIB_VXI_LA;
extern la_entry *rmtable;

	/* These globals must be initialized in the pre-init routine */
int8 BNOcommand;						/* init to 1 */
int8 OurCommander;						/* init to GPIB_VXI_CMDR */
int8 OurStatusByte;						/* init to choice */

uint32 *GPEventEnables;				/* init to 0xffffffff's */

int8 HandlerToLine[4];



HandleWordSerialPhysCommand(WSword)
	uint16 WSword;
{
	uint16 WSdata, RespEnaRet, RetModid, StatusByte, ServArea, protocols;
	uint16 eventnum, eventind;
	int8 handler, line;
	WSresp WSr;
	uint16 Our_ASYN_MODE, resphold;
	uint16 Our_RespEna, respmask;


	GPEventEnables = DCIheader.GPIB_VXI_EventEnables;

#ifdef WSPHYSCOMMANDDEBUG
		printf("\n\rPhysical WS Cmd received [0x%4x]: ",WSword);
#endif
		WSdata = WSword & 0xff;
		switch (WSword & 0xff00)
		{

			case WS_READ_STB & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Read Status Byte  ");
#endif
					/* 'OurStatusByte needs to be initialized in 'init.c' */
				WSr.Response = 0xff00 | (uint16)OurStatusByte;
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);

#ifdef WSPHYSCOMMANDDEBUG
				printf("0x%x  (ret = 0x%x)\n\r", WSr.Response, WSr.RetVal);
#endif
				break;


			case WS_TRIGGER & 0xff00:	
#ifdef WSPHYSCOMMANDDEBUG
				printf("Device Trigger.\n\r");
#endif
				break;


			case WS_CLEAR & 0xff00:	
#ifdef WSPHYSCOMMANDDEBUG
				printf("Device Clear.\n\r");
#endif
				HWwsClear(WSb_RET_Clear);

					/* clear CI device functions here */
				break;


			case WS_RD_PROT_ERR & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Read Protocol Error ");
#endif
				WSr.Response = *(DCIheader.GPIB_VXI_ProtocolError);		/* get error response */

					/* clear error condition and err* bit */
				*(DCIheader.GPIB_VXI_ProtocolError) = PROT_E_NO_ERROR;
				resphold = gpIn(Rresponse);
				gpOut(Wresponse, resphold | R_ERR);

				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("0x%x  (ret = 0x%x): ", WSr.Response, WSr.RetVal);
#endif
				break;


			case WS_ID_COMMANDER & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Identify Commander: LA = %d\n\r", WSdata);
#endif
				OurCommander = WSdata;
				DCIheader.CMDRtable[GPIB_VXI_LA] = (int8)WSdata;
				break;


			case WS_DEVGRANT & 0xff00:	
#ifdef WSPHYSCOMMANDDEBUG
				printf("Device Grant -- Logical address 0x%x. \n\r", WSdata);
#endif
/*
				DevGrant[WSdata] = 1;
				if (BNOcommand)
				{	AddDevsAfterBNO(1);
					CreateServantList(0);
				}
*/
					
				break;


			case WS_REL_DEVICE & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Release Device with la = %d\r\n", WSdata);
#endif
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("\r\n0x%x  (ret = 0x%x)\r\n", WSr.Response, WSr.RetVal);
#endif
				break;


		case WS_ANO & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
			printf("Abort Normal Operation  ");
#endif
				/* clear queues, set default state, propagate to CIs */
			WSr.Response = AbortNormalOperation();
			GPEventEnables[0] = 0xffffffff;
			GPEventEnables[1] = 0xffffffff;
			GPEventEnables[2] = 0xffffffff;
			GPEventEnables[3] = 0xffffffff;
			
			d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
			NormalOperation = 0;
			BNOcommand = 0;
#ifdef WSPHYSCOMMANDDEBUG
			printf("0x%x  (ret = 0x%x) \r\n", WSr.Response, WSr.RetVal);
#endif
			break;



		case WS_ENO & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
			printf("End Normal Operation  ");
#endif
			if (NormalOperation)
			 	WSr.Response = EndNormalOperation();
			else
			 	WSr.Response = 0x7ffe;
			
			d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
			NormalOperation = 0;
			BNOcommand = 0;
#ifdef WSPHYSCOMMANDDEBUG
			printf("0x%x  (ret = 0x%x) \r\n", WSr.Response, WSr.RetVal);
#endif
			break;




			case WS_READ_PROT & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Read Protcols  ");
#endif
				WSr.Response = rmtable[GPIB_VXI_LA].prot;
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("0x%x  (ret = 0x%x): ", WSr.Response, WSr.RetVal);
				PrintProt(WSr.Response);
#endif
				break;



			case WS_ASYN_MODE & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Asynchrous Mode Control (0x%x): ",WSdata);
#endif
				Our_ASYN_MODE = 0xfff0 | (WSdata & 0xf);
				if (!(Our_ASYN_MODE & ASYN_RESP_INT))
				{	Our_ASYN_MODE |= ASYN_RESP_ENA; 		/* 1 = disabled */
					Our_ASYN_MODE &= 0x7fff;				/* flag error, not interrupter */
				}
				if (!(Our_ASYN_MODE & ASYN_EVENT_INT))
				{	Our_ASYN_MODE |= ASYN_EVENT_ENA; 		/* 1 = disabled */
					Our_ASYN_MODE &= 0x7fff;				/* flag error, not interrupter */
				}
					/* tell commander can send through signals */
				Our_ASYN_MODE |= (ASYN_EVENT_INT | ASYN_RESP_INT);

				WSr.Response = Our_ASYN_MODE;
				rmtable[GPIB_VXI_LA].AsynMode = Our_ASYN_MODE;
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("0x%x  (ret = 0x%x)\r\n", WSr.Response, WSr.RetVal);
#endif
				break;



			case WS_RESP_ENA & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Response Enable: ");
#endif
				respmask = 0x7f & ~(RESP_DIR | RESP_DOR);
					/* Asking for an unsupported bit transition? */
				if ((WSdata & respmask) != respmask)
				{		/* error response but enable supported bits asked for */
					Our_RespEna = (0x7fff & ~(RESP_DIR | RESP_DOR)) | WSdata;
				}
				else
				{	Our_RespEna = (0xffff & ~(RESP_DIR | RESP_DOR)) | WSdata;
				}
				rmtable[GPIB_VXI_LA].RespENA = Our_RespEna;
				WSr.Response 			  = Our_RespEna;
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("0x%x  (ret = 0x%x)\r\n", WSr.Response, WSr.RetVal);
#endif
				break;



			case WS_CTRL_EVENT & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Control Event (%sable 0x%x): ", 
					(WSdata & (1<<7))? "Ena":"Dis", WSdata & 0x7f);
#endif
				eventnum = WSdata & 0x1f;
				eventind = (WSdata >> 5) & 3;

					/* enable or disable? */
				if (WSdata & (1<<7))
					GPEventEnables[eventind] |= (1<<eventnum);
				else
					GPEventEnables[eventind] &= ~(1<<eventnum);

				WSr.Response = 0xfffe;
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("0x%x  (ret = 0x%x)\r\n", WSr.Response, WSr.RetVal);
#endif
				break;



			case WS_BEGIN_NORM & 0xff00:	
#ifdef WSPHYSCOMMANDDEBUG
				printf("Begin Normal Operation. \n\r");
#endif
					/* If Not called for the first time need to propagate */
				if (BNOcommand != 0)
					BeginNormalOperation();

				if (NormalOperation)
				{	WSr.Response = 0xfffe;
					d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
				}
				else
				{ 	BNOcommand = 1;
					NormalOperation = 1;
				}
				break;


			case WS_BEGIN_NORM_A & 0xff00:	
#ifdef WSPHYSCOMMANDDEBUG
				printf("Begin Normal Operation (AUTO). \n\r");
#endif
					/* If Not called for the first time need to propagate */
				if (BNOcommand == 0)
					BeginNormalOperation();

				if (NormalOperation)
				{	WSr.Response = 0xfffe;
					d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
				}
				else
				{ 	NormalOperation = 1;
					BNOcommand = 2;
				}
				break;


			case WS_READ_SERV & 0xff00:
					/* should never happen */
				WSr.Response = (uint16)0xff00;
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
				break;

			case WS_READ_HNDLRS & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Read Handlers ");
#endif
				WSr.Response = (uint16)3;
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("0x%x  (ret = 0x%x)\r\n", WSr.Response, WSr.RetVal);
#endif
				break;


			case WS_READ_LINE & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Read Handler Line (0x%x)", WSdata);
#endif
				WSdata &= 0x07;
				if ((WSdata > 3) || (WSdata == 0))
					WSr.Response = (uint16)0x7ff8;			/* handler referenced does not exist */
				else
				{	FillHandlerToLine(HandlerToLine);
					WSr.Response = (uint16)0xfff8 | (uint16)HandlerToLine[WSdata-1];
				}
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("0x%x  (ret = 0x%x)\r\n", WSr.Response, WSr.RetVal);
#endif
				break;


			case WS_ASSIGN_HNDLR & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Assign Handler (0x%x) \r\n", WSword);
#endif
				FillHandlerToLine(HandlerToLine);
				handler = (WSdata & 0x70) >> 4;
				line = WSdata & 0x07;
				switch (handler)
				{   case 1:     
						SetIntHandlers(line, HandlerToLine[1], HandlerToLine[2]);
						WSr.Response = (uint16)0xfffe;
						break;
					case 2:     
						SetIntHandlers(HandlerToLine[0], line, HandlerToLine[2]);
						WSr.Response = (uint16)0xfffe;
						break;
					case 3:     
						SetIntHandlers(HandlerToLine[0], HandlerToLine[1], line);
						WSr.Response = (uint16)0xfffe;
						break;

					default:
						WSr.Response = (uint16)0x7ffe;	/* non-existent handler */
						break;
				}
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
				break;


			case WS_READ_MODID & 0xff00:
#ifdef WSPHYSCOMMANDDEBUG
				printf("Read MODID: ");
#endif
				if (rmtable[GPIB_VXI_LA].slot == 0)
					WSr.Response = 0xc000 | gpIn(modid);
				else
					WSr.Response = 0;
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
				
#ifdef WSPHYSCOMMANDDEBUG
				printf("0x%x  (ret = 0x%x)\r\n", WSr.Response, WSr.RetVal);
#endif
				break;

			case WS_UPPER_MODID & 0xff00:	
				RetModid = 0;
				if (rmtable[GPIB_VXI_LA].slot == 0)
				{ 	if (WSdata & WS_MODID_ENA) 
					{ 	RetModid = gpIn(modid);
						if (RetModid & M_ENABLE)
							RetModid &= (M_ENABLE | M_MODID_00_06);
						else
							RetModid = 0;
						RetModid = M_ENABLE | ((WSdata & ~WS_MODID_ENA) << 7);
					}
					else
						RetModid = 0;
					gpOut(modid, RetModid);
					WSr.Response = 0xfffe;
				}
				else
				{ 	WSr.Response = 0x7ffe;
				}
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("Set Upper Modid (0x%2x) --> 0x%4x. Response = 0x%x\n\r", 
					WSdata, RetModid, WSr.Response);
#endif
				break;


			case WS_LOWER_MODID & 0xff00:	
				RetModid = 0;
				if (rmtable[GPIB_VXI_LA].slot == 0)
				{ 	if (WSdata & WS_MODID_ENA) 
					{ 	RetModid = gpIn(modid);
						if (RetModid & M_ENABLE)
							RetModid &= (M_ENABLE | M_MODID_07_12);
						else
							RetModid = 0;
						RetModid = M_ENABLE | (WSdata & ~WS_MODID_ENA);
					}
					else
						RetModid = 0;
					gpOut(modid, RetModid);
					WSr.Response = 0xfffe;
				}
				else
				{ 	WSr.Response = 0x7ffe;
				}
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_Response,&WSr);
#ifdef WSPHYSCOMMANDDEBUG
				printf("Set Lower Modid (0x%2x) --> 0x%4x. Response = 0x%x\n\r", 
					WSdata, RetModid, WSr.Response);
#endif
				break;


				/* 
				 * buffer transfers and DIR/DOR manipulation are handled automatically
				 * by the driver. If we receive a BAV or BRQ we must NOT have been
				 * DIR or DOR (respectively).
				 */
			case WS_BYTERQ & 0xff00:	
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_GenProtErr,PROT_E_DOR_VIOL);
				break;
			case WS_BYTEAV_0 & 0xff00:	
			case WS_BYTEAV_1 & 0xff00:	
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_GenProtErr,PROT_E_DIR_VIOL);
				break;


			default:	
#ifdef WSPHYSCOMMANDDEBUG
				if (WSword & 0x8000)
				{
					printf("VXIbus reserved command --> 0x%x\n\r", WSword);
				}
				else
				{
					printf("User defined command --> 0x%x\n\r", WSword);
				}
#endif

					/* Generate Unsupported command Protocol Error */
				d_ctrl(DEV_WSsl,GPIB_VXI_LA,WSsl_CTRL_GenProtErr,PROT_E_UNREC_COM);	

				break;
		}

			/* Write Ready MUST stay clear until either response sent or Protocol error */
			/* Done, so set Write Ready in order to process the next command */
		resphold = gpIn(Rresponse);
		gpOut(Wresponse, resphold | R_WREADY);
}	



/*	
 *	The function is used to take over the physical Word Serial port of the
 * GPIB-VXI.  Upon successful return, all Word Serial commands will be routed
 * to the Asynch process exchange.  Driver calls for buffer transfers must be 
 * made in order to initiate any buffer transfers (there is no DORrequest
 * signal for the physical Word Serial port, it is not the GPIB).
 */
TakeOverWS()
{
	int8 Pcom[30];
	CommonBuf *p;

			/* Create Command string for GPIB-VXI Local Command Set Parser */
		sprintf(Pcom,"wordserena %d", MyLA);

		if ((p = DCIParserQuery(ParserId, Pcom )) != 0)
		{		
			free_seg(p); 					/* free up error return buffer */
			printf("ERROR-> Unable to take over physical Word Serial port \r\n");
			return 0;
		}
	return 1;
}


#define PWS_DEBUG 1

/* The following is the process to handle Word Serial entry to the Local Command
 * set.
 */
PhysicalWSread()
{
	uint32 count, WSend;
	CommonBuf *WSBuf;
	WSbuf WSb;

#ifdef PWS_DEBUG
start("0.0","0.0","0.0");
#endif

StartInOver:
	for (;;)
	{		
		WSend = 0;
		while (!WSend)
		{ 		
			WSb.mode = 0;
			WSb.MaxReadCount = 512;
			WSb.Wait = 0L; 			/* wait forever */
        	d_read(DEV_WSsl,GPIB_VXI_LA,&WSb,&WSBuf,0);

#ifdef PWS_DEBUG
printf("Physical WS Read done (ret == 0x%x)\n\r",WSb.RetVal);
#endif

			if (!(WSb.RetVal & WSb_RET_IODONE))
			{	pause_p(5);
#ifdef PWS_DEBUG
printf("Physical WS Read ERROR...start over\r\n");
#endif
				goto StartInOver;
			}

			WSBuf->data[WSBuf->byteCount] = '\0';

#ifdef PWS_DEBUG
if (WSBuf->byteCount > 50)
	printf("WSpI -> WS Parser Input: %d bytes received (no print it)\n\r",WSBuf->byteCount);
else
	printf("WSpI -> WS Parser Input: %d bytes received:\n\r%s\n\r",WSBuf->byteCount,WSBuf->data);
#endif

			WSend = (WSBuf->EndFlag)? 1 : 0;
	
			free_seg(WSBuf);
		}
	}
}


PhysicalWSwrite()
{
	uint32 count, WSend;
	CommonBuf *WSBuf;
	WSbuf WSb;


	WSb.mode = 0;
	d_write(DEV_WSsl,GPIB_VXI_LA,&WSb,&WSBuf,0);

	if (!(WSb.RetVal & WSb_RET_IODONE))
	{		
	}
}
		


