
/*****************************************************************************/
/*                                                                           */
/*                  National Instruments GPIB-VXI Project                    */
/*                       DownLoable Code Instruments                         */
/*                                                                           */
/*                     (C) 1992  National Instruments                        */
/*                                                                           */
/*****************************************************************************/

#include "stdio.h"
#include "DataSize.h"
#include "ResManIn.h"
#include "S_EVENT.h"
#include "CommonBu.h"
#include "InclWS.h"
#include "DciStruc.h"
#include "DciFuncs.h"
#include "UserDMA.h"
#include "signals.h"


	/* Quick reference variables */
la_entry *rmtable;
int8 MyLA, Cmdr, GPIB_VXI_LA;
uint16 ParserId;
int8 SecondaryAddress;
extern uint16 *VXIregs;
extern int8 SoftReset;
extern uint32 CIEventEnables[4];

	/* Keeps track of number of Word Serial Data In Events */
extern int32 WS_DATA_IN_NUM;
extern int8 GetOutput;
extern 

/*
 *  The purpose of the Worker process is to concentrate on the device
 * specific operation of the DCI.  The Asynch process handles most of the
 * VXI protocols and notifies other processes (such as the Worker process)
 * only in the case of the event affecting its operation.
 *	The Worker process is meant to simulate a VXI Message Based device's
 * power-on and runtime activities.  It has time to set its PASSED bit in
 * its STATUS register and MUST wait for Begin Normal Operation to occur 
 * (the Asynch process will notify it of that fact).  It may then go about
 * performing any I/O and other operations it sees fit to execute within
 * the bounds of the VXI Specification.  Basically, the worker process can
 * do just about anything.... It can take buffers in off the GPIB, put buffers
 * on the GPIB, create a local command set of its own, access Register based
 * devices, access Message Based devices, access foreign devices, configure
 * and re-configure the VXI system, etc.
 */

void WorkerProcess()
{
	uint16 caught;

	
	start("0.0","0.0","0.0"); 			/* Must call to use UART (Serial Port) I/O */

		/* Initialize global variables if cold booting */
	if (!SoftReset)
		InitGlobals();

	/************************************/
	/* 	Perform any self-test or other operations needed before beginning
	 * operation HERE.  Then set your PASSED bit.
	 */
	/************************************/

		/* Set Passed bit and Not Ready bit */
	VXIregs[StatusReg] |= Status_PASSED;

		/* NOTE ----> do not Call GPIB-VXI Local Parser until BNO is received */
		/* Wait for a Begin Normal Operation to occur */
	wait_v(S_EVENT_BNO,1,0,&caught);

	/*******************************************************/
	/* DO ANY POST Begin Normal Operation preparation HERE */
	/*******************************************************/

		/* If cold booting set Secondary Address (otherwise is already set) */
	if (!SoftReset)
		SetSecondaryAddress();

	MainLoop();
	suspend_p(0);
}


void Handler()
{
	printf("Handler Starting\r\n");

	while(1)
	{		
	}
}

uint32 HNDpid;
int8 DebugMode;

MainLoop()
{
	uint32 source, dest, count, mode, wait;
	int8 ret, rt, endflag;
	uint16 stat,caught;
	CommonBuf *WSBuf, *Outbuf;
	WSbuf WSb;
	WScom WSc;

	printf("\r\nNational Instrument's DMAmove Code Instrument Running\r\n");
/*	
	spawn_p("IHAN",0,1,0x100,0x100,0,&HNDpid);	/* Start a VXI IRQHandler 
	printf("HNDpid = %d\r\n",HNDpid);
	printf("Handler Addr = 0x%x\r\n", Handler);
	printf("WorkerStart Addr = 0x%x\r\n", DCIheader.WorkerStart);

	activate_p(HNDpid, DCIheader.WorkerStart);
*/
	

	DebugMode = DCIheader.Debug? 1 : 0;
	for(;;)
	{ 		
			/* if enabled for DIR response interrupts send one */
		if (!(DCIheader.RespEna & RESP_DIR))		/* active low */
			SignalOut(Cmdr, (((RESP_DIR | RESP_FHS | RESP_ERR)<<8) | MyLA));

		if (!(DCIheader.StatusByte & 0x40))
			DCIheader.StatusByte = 0xff00;

			/* wait for a Word Serial Buffer In */
		wait_v(S_EVENT_DCI_IO,1,0,&caught);

		while (WS_DATA_IN_NUM > 0)
		{
			if (!(DCIheader.StatusByte & 0x40))
				DCIheader.StatusByte = 0xff80;

			WSb.mode = 0;
			WSb.MaxReadCount = 0x200;
			WSb.Wait = 0L; 		         /* wait forever for buffer allocation */
				/* Read in the actual buffer */
			d_read(DEV_WSsl,MyLA,&WSb,&WSBuf,0);
			if (!(WSb.RetVal & WSb_RET_IODONE))
			{	printf("D_READ ERROR returns 0x%x \r\n",WSb.RetVal);
				continue;
			}

				/* get DMAmove parameters from buffer */
			source = *(uint32 *)(&(WSBuf->data[0]));
			dest   = *(uint32 *)(&(WSBuf->data[4]));
			count  = *(uint32 *)(&(WSBuf->data[8]));
			mode   = *(uint32 *)(&(WSBuf->data[12]));

				/* MUST FREE BUFFER in the DCI */
			rt = free_seg(WSBuf);
			if (rt != 0x00)
				printf("Freeing error for WSbuf (Inputted buffer).\r\n");
			WS_DATA_IN_NUM--;	/* decrement the one buffer just used from the total */

			if ((WSBuf->byteCount != 16) || (source == 0 && dest == 0) || (count > 0x800000) ||
				(source > 0xffffff) || (dest > 0xffffff))
			{	
				DCIheader.StatusByte = 0xff50;
				SignalOut(Cmdr, SIG_REQT | MyLA);

				printf ("\n\r");
				printf ("DMAmove CI: Error in parameters sent: \n\r");
				printf ("  source = 0x%8x    dest   = 0x%8x \n\r", source, dest);
				printf ("  count  = 0x%8x    mode   = 0x%8x \n\r", count, mode);
				printf ("\n\r");
				printf ("Function: DMAmove(source, dest, count, mode) \n\r");
				printf ("  Format Over GPIB (motorola byte order):\n\r");
				printf ("    byte 0-3:  source address (<= 0x00ffffff, 0=GPIB input  w/ dest != 0)\n\r");
				printf ("    byte 4-7:  dest address   (<= 0x00ffffff, 0=GPIB output w/ source != 0)\n\r");
				printf ("    byte 8-B:  count          (in bytes <= 0x00800000)\n\r");
				printf ("    byte C-F:  mode           (0 = 16 bit, inc src+dest, cycle steal)\n\r");
				printf ("\n\r");
			}
			else
			{
				if (DebugMode)
				{	printf ("DMAmove CI: Parameters sent: \n\r");
					printf ("  source = 0x%8x    dest   = 0x%8x \n\r", source, dest);
					printf ("  count  = 0x%8x    mode   = 0x%8x \n\r", count, mode);
					printf ("\n\r");
				}
				ret = 0;
				if (source == 0)	/* move from GPIB to memory somewhere */
				{
						/* wait for data buffer to arrive */
					for (wait = 0; WS_DATA_IN_NUM == 0 && wait < 1000; wait++)
						pause_p(1);
					WSb.mode = 0;
					WSb.MaxReadCount = 0x200;
					WSb.Wait = 0L; 		         /* wait forever for buffer allocation */
						/* Read in the actual buffer */
					d_read(DEV_WSsl,MyLA,&WSb,&WSBuf,0);
					if (!(WSb.RetVal & WSb_RET_IODONE))
					{	printf("Buffer read from GPIB ERROR (returns 0x%x): 0x200 = NO DATA \r\n",WSb.RetVal);
						continue;
					}

					for (;;)
					{	source = (uint32)(&(WSBuf->data[0]));
						if (DebugMode)
						{	printf("Do DMAmove(GPIB bufffer: 0x%x, 0x%x, 0x%x, %d) \n\r",
								source,dest,WSBuf->byteCount,mode);
						}
						ret = DMAmove(source, dest, WSBuf->byteCount, mode);

							if (ret != 0 || (WSBuf->EndFlag))
						{
							rt = free_seg(WSBuf);
							if (rt != 0x00)
								printf("Freeing error for WSbuf (Inputted buffer).\r\n");
							WS_DATA_IN_NUM--;	/* decrement the one buffer just used from the total */
							break;
						}

						if (!(mode & DMA_MODE_NoIncDEST))
							dest += WSBuf->byteCount;

						rt = free_seg(WSBuf);
						if (rt != 0x00)
							printf("Freeing error for WSbuf (Inputted buffer).\r\n");
						WS_DATA_IN_NUM--;	/* decrement the one buffer just used from the total */

							/* wait for data buffer to arrive */
						for (wait = 0; WS_DATA_IN_NUM == 0 && wait < 1000; wait++)
							pause_p(1);

						WSb.mode = 0;
						WSb.MaxReadCount = 0x200;
						WSb.Wait = 0L; 		         /* wait forever for buffer allocation */
							/* Read in the actual buffer */
						d_read(DEV_WSsl,MyLA,&WSb,&WSBuf,0);
						if (!(WSb.RetVal & WSb_RET_IODONE))
						{	printf("D_READ ERROR returns 0x%x \r\n",WSb.RetVal);
							continue;
						}
					
					}

				}
				else if (dest == 0)		/* move from somewhere in memory to GPIB */
				{
					for (;;)
					{	if (count == 0)
							break; 
#define BUF_SIZE 0x8000
						if (count > BUF_SIZE)
						{	WSBuf = CreateBuf((BUF_SIZE >> 8), 0);
							WSBuf->byteCount = BUF_SIZE;
							WSBuf->EndFlag = 0;
							count -= BUF_SIZE;
						}
						else
						{	WSBuf = CreateBuf((count >> 8), 0);
							WSBuf->byteCount = count;
							WSBuf->EndFlag = 1;
							count = 0;
						}
						if (DebugMode)
						{ 	printf("Do DMAmove(0x%x, GPIB buffer: 0x%x, 0x%x, %d) \n\r",
								source,(uint32)(&(WSBuf->data[0])),WSBuf->byteCount,mode);
						}
						ret = DMAmove (source, (uint32)(&(WSBuf->data[0])), WSBuf->byteCount, mode);
	
						if (!(mode & DMA_MODE_NoIncSRC))
							source += WSBuf->byteCount;

						if (ret != 0)
						{
							rt = free_seg(WSBuf);
							if (rt != 0x00)
								printf("Freeing error for WSbuf (Outputted buffer).\r\n");
							break;
						}

						WSb.mode = 0;
						d_write(DEV_WSsl,MyLA,&WSb,&WSBuf,0);
		
							/* if error sending buffer, throw it away */
						if (!(WSb.RetVal & WSb_RET_IODONE))
						{	/*printf("D_WRITE ERROR returns 0x%x \r\n",WSb.RetVal);*/
							free_seg(WSBuf);
							continue;
						}
					}
				}
				else
				{
					if (DebugMode)
					{
						printf("Do DMAmove(0x%x,0x%x,0x%x,%d) \n\r",
							source,dest,count,mode);
					}
					ret = DMAmove(source, dest, count, mode);
				}

				if (ret != 0)
				{	
					switch (ret)
					{
					case 2:		printf ("DMA timing error occurred \r\n");
								DCIheader.StatusByte = 0xff41;
								break;
					case 9:		printf ("Bus Error occurred at source \r\n");
								DCIheader.StatusByte = 0xff42;
								break;
					case 10:	printf ("Bus Error occurred at destination \r\n");
								DCIheader.StatusByte = 0xff44;
								break;
					default:	printf ("Unknown Error. \r\n");
								DCIheader.StatusByte = 0xff48;
								break;
					}
					SignalOut(Cmdr, SIG_REQT | MyLA);
				}
				else
				{
					if (DebugMode)
						printf ("Transfer successful \r\n");
				}

			}

		}

	}
}




InitGlobals()
{
		/* Get My Logical Address, Commanders and the GPIB-VXI's (if needed later) */
	MyLA 		= DCIheader.DCILogicalAddress; 
	GPIB_VXI_LA = DCIheader.GPIB_VXI_LA;
	Cmdr 		= DCIheader.CmdrLogicalAddress;

	ParserId	= DCIheader.ParserId;
	rmtable		= &DCIheader.la_table[0];

	if (!SoftReset)
	{		/* active low:  globally enabled for reponses/events */
		DCIheader.AsynMode  =  0xffff & ~(ASYN_RESP_ENA | ASYN_EVENT_ENA);
		rmtable[MyLA].AsynMode   =  0xffff & ~(ASYN_RESP_ENA | ASYN_EVENT_ENA);
			/* active low: not enabled for any responses (until commander tells us) */
		DCIheader.RespEna   = 0xffff;
		rmtable[MyLA].RespENA    = 0xffff;
			/* enable all bit-wise event enable masks */
		CIEventEnables[0] = 0xffffffff;
		CIEventEnables[1] = 0xffffffff;
		CIEventEnables[2] = 0xffffffff;
		CIEventEnables[3] = 0xffffffff;
	}
		/* no Secondary Address set yet */
	SecondaryAddress = 255;
}




/*	
 *	This function sends a command to the GPIB-VXI Local Command Set in order
 * to set this CI's Secondary Address.  It tries to set the Secondary Address
 * to the upper five bits of the Logical Address but if it can't it finds the
 * next available Secondary Address (if any).
 */

SetSecondaryAddress()
{
	int8 Pcom[30], SecAddr;
	CommonBuf *p;

		/* Secondary address is upper 5 bits of Logical address */
	SecAddr = (MyLA >> 3);

	for (;;)
	{
			/* Create Command string for GPIB-VXI Local Command Set Parser */
		sprintf(Pcom,"lasaddr %d,%d", MyLA, SecAddr);

			/* Since 'LaSaddr' has no response, response will occur only if error */
		if ((p = DCIParserQuery(ParserId, Pcom)) != 0)
		{		
			free_seg(p); 					/* free up error return buffer */
			SecAddr++;						/* try next Secodary Address */
			if (SecAddr == (MyLA >> 3))		/* if got back to first one, give up */
				return 0;
			if (SecAddr > 0x1e)				/* wrap around */
				SecAddr = 0;
		}
		else
			break;							/* Secondary Address set, so leave */
	}

		/* copy new Secondary Address to global variable */
	SecondaryAddress = SecAddr;

	return 1;
}


